/* Generated by Cython 0.23.3 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [], 
        "language": "c++"
    }
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_3"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) (void_result, Py_INCREF(Py_None), Py_None)

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #define CYTHON_INLINE inline
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    operator T&() { return *ptr; }
  private:
    T *ptr;
};

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__centrosome___filter
#define __PYX_HAVE_API__centrosome___filter
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "centrosome/_filter.pyx",
  "__init__.pxd",
  "type.pxd",
};
#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;


/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;

/* "centrosome/_filter.pyx":45
 * DTYPE_UINT32 = np.uint32
 * DTYPE_BOOL = np.bool
 * ctypedef np.uint16_t pixel_count_t             # <<<<<<<<<<<<<<
 * 
 * ###########
 */
typedef __pyx_t_5numpy_uint16_t __pyx_t_10centrosome_7_filter_pixel_count_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_t_10centrosome_7_filter_HistogramPiece;
struct __pyx_t_10centrosome_7_filter_Histogram;
struct __pyx_t_10centrosome_7_filter_PixelCount;
struct __pyx_t_10centrosome_7_filter_SCoord;
struct __pyx_t_10centrosome_7_filter_Histograms;

/* "centrosome/_filter.pyx":59
 * ###########
 * 
 * cdef struct HistogramPiece:             # <<<<<<<<<<<<<<
 *     np.uint16_t coarse[16]
 *     np.uint16_t fine[256]
 */
struct __pyx_t_10centrosome_7_filter_HistogramPiece {
  __pyx_t_5numpy_uint16_t coarse[16];
  __pyx_t_5numpy_uint16_t fine[0x100];
};

/* "centrosome/_filter.pyx":63
 *     np.uint16_t fine[256]
 * 
 * cdef struct Histogram:             # <<<<<<<<<<<<<<
 *     HistogramPiece top_left     # top-left corner
 *     HistogramPiece top_right    # top-right corner
 */
struct __pyx_t_10centrosome_7_filter_Histogram {
  struct __pyx_t_10centrosome_7_filter_HistogramPiece top_left;
  struct __pyx_t_10centrosome_7_filter_HistogramPiece top_right;
  struct __pyx_t_10centrosome_7_filter_HistogramPiece edge;
  struct __pyx_t_10centrosome_7_filter_HistogramPiece bottom_left;
  struct __pyx_t_10centrosome_7_filter_HistogramPiece bottom_right;
};

/* "centrosome/_filter.pyx":74
 * # because of the mask
 * #
 * cdef struct PixelCount:             # <<<<<<<<<<<<<<
 *     pixel_count_t top_left
 *     pixel_count_t top_right
 */
struct __pyx_t_10centrosome_7_filter_PixelCount {
  __pyx_t_10centrosome_7_filter_pixel_count_t top_left;
  __pyx_t_10centrosome_7_filter_pixel_count_t top_right;
  __pyx_t_10centrosome_7_filter_pixel_count_t edge;
  __pyx_t_10centrosome_7_filter_pixel_count_t bottom_left;
  __pyx_t_10centrosome_7_filter_pixel_count_t bottom_right;
};

/* "centrosome/_filter.pyx":85
 * # relative offsets from the octagon center
 * #
 * cdef struct SCoord:             # <<<<<<<<<<<<<<
 *     np.int32_t stride   # add the stride to the memory location
 *     np.int32_t x
 */
struct __pyx_t_10centrosome_7_filter_SCoord {
  __pyx_t_5numpy_int32_t stride;
  __pyx_t_5numpy_int32_t x;
  __pyx_t_5numpy_int32_t y;
};

/* "centrosome/_filter.pyx":90
 *     np.int32_t y
 * 
 * cdef struct Histograms:             # <<<<<<<<<<<<<<
 *     void *memory                # pointer to the allocated memory
 *     Histogram *histogram        # pointer to the histogram memory
 */
struct __pyx_t_10centrosome_7_filter_Histograms {
  void *memory;
  struct __pyx_t_10centrosome_7_filter_Histogram *histogram;
  struct __pyx_t_10centrosome_7_filter_PixelCount *pixel_count;
  __pyx_t_5numpy_uint8_t *data;
  __pyx_t_5numpy_uint8_t *mask;
  __pyx_t_5numpy_uint8_t *output;
  __pyx_t_5numpy_int32_t column_count;
  __pyx_t_5numpy_int32_t stripe_length;
  __pyx_t_5numpy_int32_t row_count;
  __pyx_t_5numpy_int32_t current_column;
  __pyx_t_5numpy_int32_t current_row;
  __pyx_t_5numpy_int32_t current_stride;
  __pyx_t_5numpy_int32_t radius;
  __pyx_t_5numpy_int32_t a_2;
  struct __pyx_t_10centrosome_7_filter_SCoord last_top_left;
  struct __pyx_t_10centrosome_7_filter_SCoord top_left;
  struct __pyx_t_10centrosome_7_filter_SCoord last_top_right;
  struct __pyx_t_10centrosome_7_filter_SCoord top_right;
  struct __pyx_t_10centrosome_7_filter_SCoord last_leading_edge;
  struct __pyx_t_10centrosome_7_filter_SCoord leading_edge;
  struct __pyx_t_10centrosome_7_filter_SCoord last_bottom_right;
  struct __pyx_t_10centrosome_7_filter_SCoord bottom_right;
  struct __pyx_t_10centrosome_7_filter_SCoord last_bottom_left;
  struct __pyx_t_10centrosome_7_filter_SCoord bottom_left;
  __pyx_t_5numpy_int32_t row_stride;
  __pyx_t_5numpy_int32_t col_stride;
  struct __pyx_t_10centrosome_7_filter_HistogramPiece accumulator;
  __pyx_t_5numpy_uint32_t accumulator_count;
  __pyx_t_5numpy_int32_t percent;
  __pyx_t_5numpy_int32_t last_update_column[16];
};

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static CYTHON_INLINE long __Pyx_div_long(long, long);

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

static CYTHON_INLINE long __Pyx_mod_long(long, long);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

static CYTHON_INLINE int  __Pyx_GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

static void __Pyx_RaiseBufferFallbackError(void);

static CYTHON_INLINE npy_intp __Pyx_div_npy_intp(npy_intp, npy_intp);

#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


static Py_ssize_t __Pyx_zeros[] = {0, 0, 0, 0, 0, 0, 0, 0};
static Py_ssize_t __Pyx_minusones[] = {-1, -1, -1, -1, -1, -1, -1, -1};

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE npy_int32 __Pyx_PyInt_As_npy_int32(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint8(npy_uint8 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'centrosome._filter' */
static PyTypeObject *__pyx_ptype_10centrosome_7_filter_ndarray = 0;
static struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_f_10centrosome_7_filter_allocate_histograms(__pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_uint8_t *, __pyx_t_5numpy_uint8_t *, __pyx_t_5numpy_uint8_t *); /*proto*/
static void __pyx_f_10centrosome_7_filter_free_histograms(struct __pyx_t_10centrosome_7_filter_Histograms *); /*proto*/
static void __pyx_f_10centrosome_7_filter_set_stride(struct __pyx_t_10centrosome_7_filter_Histograms *, struct __pyx_t_10centrosome_7_filter_SCoord *); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_tl_br_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_tr_bl_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_leading_edge_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_trailing_edge_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_add16(__pyx_t_5numpy_uint16_t *, __pyx_t_5numpy_uint16_t *); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_sub16(__pyx_t_5numpy_uint16_t *, __pyx_t_5numpy_uint16_t *); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate_coarse_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_deaccumulate_coarse_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate_fine_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_uint32_t); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_deaccumulate_fine_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_uint32_t); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate(struct __pyx_t_10centrosome_7_filter_Histograms *); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_fine(struct __pyx_t_10centrosome_7_filter_Histograms *, int); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *, struct __pyx_t_10centrosome_7_filter_HistogramPiece *, __pyx_t_10centrosome_7_filter_pixel_count_t *, struct __pyx_t_10centrosome_7_filter_SCoord *, struct __pyx_t_10centrosome_7_filter_SCoord *); /*proto*/
static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_current_location(struct __pyx_t_10centrosome_7_filter_Histograms *); /*proto*/
static CYTHON_INLINE __pyx_t_5numpy_uint8_t __pyx_f_10centrosome_7_filter_find_median(struct __pyx_t_10centrosome_7_filter_Histograms *); /*proto*/
static int __pyx_f_10centrosome_7_filter_c_median_filter(__pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_int32_t, __pyx_t_5numpy_uint8_t *, __pyx_t_5numpy_uint8_t *, __pyx_t_5numpy_uint8_t *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t = { "uint8_t", NULL, sizeof(__pyx_t_5numpy_uint8_t), { 0 }, 0, IS_UNSIGNED(__pyx_t_5numpy_uint8_t) ? 'U' : 'I', IS_UNSIGNED(__pyx_t_5numpy_uint8_t), 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t = { "float64_t", NULL, sizeof(__pyx_t_5numpy_float64_t), { 0 }, 0, 'R', 0, 0 };
#define __Pyx_MODULE_NAME "centrosome._filter"
int __pyx_module_is_main_centrosome___filter = 0;

/* Implementation of 'centrosome._filter' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_RuntimeError;
static char __pyx_k_B[] = "B";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_a[] = "a";
static char __pyx_k_b[] = "b";
static char __pyx_k_c[] = "c";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_j[] = "j";
static char __pyx_k_k[] = "k";
static char __pyx_k_l[] = "l";
static char __pyx_k_p[] = "p";
static char __pyx_k_q[] = "q";
static char __pyx_k_x[] = "x";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_ik[] = "ik";
static char __pyx_k_jk[] = "jk";
static char __pyx_k_np[] = "np";
static char __pyx_k_pa[] = "pa";
static char __pyx_k_pb[] = "pb";
static char __pyx_k_pc[] = "pc";
static char __pyx_k_ptr[] = "ptr";
static char __pyx_k_bool[] = "bool";
static char __pyx_k_data[] = "data";
static char __pyx_k_imax[] = "imax";
static char __pyx_k_jmax[] = "jmax";
static char __pyx_k_kmax[] = "kmax";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_mask[] = "mask";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_dtype[] = "dtype";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_pmask[] = "pmask";
static char __pyx_k_range[] = "range";
static char __pyx_k_uint8[] = "uint8";
static char __pyx_k_zeros[] = "zeros";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_kernel[] = "kernel";
static char __pyx_k_output[] = "output";
static char __pyx_k_pimage[] = "pimage";
static char __pyx_k_radius[] = "radius";
static char __pyx_k_uint32[] = "uint32";
static char __pyx_k_istride[] = "istride";
static char __pyx_k_kstride[] = "kstride";
static char __pyx_k_mstride[] = "mstride";
static char __pyx_k_percent[] = "percent";
static char __pyx_k_pkernel[] = "pkernel";
static char __pyx_k_poutput[] = "poutput";
static char __pyx_k_big_mask[] = "big_mask";
static char __pyx_k_estimate[] = "estimate";
static char __pyx_k_DTYPE_BOOL[] = "DTYPE_BOOL";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_MemoryError[] = "MemoryError";
static char __pyx_k_accumulator[] = "accumulator";
static char __pyx_k_mask_offset[] = "mask_offset";
static char __pyx_k_mask_stride[] = "mask_stride";
static char __pyx_k_DTYPE_UINT32[] = "DTYPE_UINT32";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_image_stride[] = "image_stride";
static char __pyx_k_kernel_width[] = "kernel_width";
static char __pyx_k_pixel_stride[] = "pixel_stride";
static char __pyx_k_plane_number[] = "plane_number";
static char __pyx_k_raster_count[] = "raster_count";
static char __pyx_k_kernel_stride[] = "kernel_stride";
static char __pyx_k_median_filter[] = "median_filter";
static char __pyx_k_paeth_decoder[] = "paeth_decoder";
static char __pyx_k_raster_number[] = "raster_number";
static char __pyx_k_raster_stride[] = "raster_stride";
static char __pyx_k_kernel_half_width[] = "kernel_half_width";
static char __pyx_k_centrosome__filter[] = "centrosome._filter";
static char __pyx_k_masked_convolution[] = "masked_convolution";
static char __pyx_k_Kernel_must_be_square[] = "Kernel must be square";
static char __pyx_k_Kernel_shape_must_be_odd[] = "Kernel shape must be odd";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_Median_filter_percent_d_is_less[] = "Median filter percent = %d is less than zero";
static char __pyx_k_Users_agoodman_github_CellProfi[] = "/Users/agoodman/github/CellProfiler/centrosome/centrosome/_filter.pyx";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_Data_shape_d_d_is_not_mask_shape[] = "Data shape (%d,%d) is not mask shape (%d,%d)";
static char __pyx_k_Data_shape_d_d_is_not_output_sha[] = "Data shape (%d,%d) is not output shape (%d,%d)";
static char __pyx_k_Failed_to_allocate_scratchpad_me[] = "Failed to allocate scratchpad memory";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Median_filter_percent_d_is_great[] = "Median filter percent = %d is greater than 100";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_n_s_DTYPE_BOOL;
static PyObject *__pyx_n_s_DTYPE_UINT32;
static PyObject *__pyx_kp_s_Data_shape_d_d_is_not_mask_shape;
static PyObject *__pyx_kp_s_Data_shape_d_d_is_not_output_sha;
static PyObject *__pyx_kp_s_Failed_to_allocate_scratchpad_me;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_s_Kernel_must_be_square;
static PyObject *__pyx_kp_s_Kernel_shape_must_be_odd;
static PyObject *__pyx_kp_s_Median_filter_percent_d_is_great;
static PyObject *__pyx_kp_s_Median_filter_percent_d_is_less;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_kp_s_Users_agoodman_github_CellProfi;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_accumulator;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_big_mask;
static PyObject *__pyx_n_s_bool;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_s_centrosome__filter;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_estimate;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_ik;
static PyObject *__pyx_n_s_image_stride;
static PyObject *__pyx_n_s_imax;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_istride;
static PyObject *__pyx_n_s_j;
static PyObject *__pyx_n_s_jk;
static PyObject *__pyx_n_s_jmax;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_kernel;
static PyObject *__pyx_n_s_kernel_half_width;
static PyObject *__pyx_n_s_kernel_stride;
static PyObject *__pyx_n_s_kernel_width;
static PyObject *__pyx_n_s_kmax;
static PyObject *__pyx_n_s_kstride;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mask;
static PyObject *__pyx_n_s_mask_offset;
static PyObject *__pyx_n_s_mask_stride;
static PyObject *__pyx_n_s_masked_convolution;
static PyObject *__pyx_n_s_median_filter;
static PyObject *__pyx_n_s_mstride;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_output;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_pa;
static PyObject *__pyx_n_s_paeth_decoder;
static PyObject *__pyx_n_s_pb;
static PyObject *__pyx_n_s_pc;
static PyObject *__pyx_n_s_percent;
static PyObject *__pyx_n_s_pimage;
static PyObject *__pyx_n_s_pixel_stride;
static PyObject *__pyx_n_s_pkernel;
static PyObject *__pyx_n_s_plane_number;
static PyObject *__pyx_n_s_pmask;
static PyObject *__pyx_n_s_poutput;
static PyObject *__pyx_n_s_ptr;
static PyObject *__pyx_n_s_radius;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_raster_count;
static PyObject *__pyx_n_s_raster_number;
static PyObject *__pyx_n_s_raster_stride;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_uint32;
static PyObject *__pyx_n_s_uint8;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_10centrosome_7_filter_median_filter(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_data, PyArrayObject *__pyx_v_mask, PyArrayObject *__pyx_v_output, int __pyx_v_radius, __pyx_t_5numpy_int32_t __pyx_v_percent); /* proto */
static PyObject *__pyx_pf_10centrosome_7_filter_2masked_convolution(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_data, PyArrayObject *__pyx_v_mask, PyArrayObject *__pyx_v_kernel); /* proto */
static PyObject *__pyx_pf_10centrosome_7_filter_4paeth_decoder(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, __pyx_t_5numpy_int32_t __pyx_v_raster_count); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__13;

/* "centrosome/_filter.pyx":166
 * #
 * ############################################################################
 * cdef Histograms *allocate_histograms(np.int32_t   rows,             # <<<<<<<<<<<<<<
 *                                             np.int32_t   columns,
 *                                             np.int32_t   row_stride,
 */

static struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_f_10centrosome_7_filter_allocate_histograms(__pyx_t_5numpy_int32_t __pyx_v_rows, __pyx_t_5numpy_int32_t __pyx_v_columns, __pyx_t_5numpy_int32_t __pyx_v_row_stride, __pyx_t_5numpy_int32_t __pyx_v_col_stride, __pyx_t_5numpy_int32_t __pyx_v_radius, __pyx_t_5numpy_int32_t __pyx_v_percent, __pyx_t_5numpy_uint8_t *__pyx_v_data, __pyx_t_5numpy_uint8_t *__pyx_v_mask, __pyx_t_5numpy_uint8_t *__pyx_v_output) {
  unsigned int __pyx_v_adjusted_stripe_length;
  unsigned int __pyx_v_memory_size;
  void *__pyx_v_ptr;
  struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph;
  size_t __pyx_v_roundoff;
  int __pyx_v_a;
  PyObject *__pyx_v_a_2 = NULL;
  struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __pyx_t_5numpy_int32_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("allocate_histograms", 0);

  /* "centrosome/_filter.pyx":176
 *                                             np.uint8_t * output):
 *     cdef:
 *         unsigned int adjusted_stripe_length = columns + 2*radius + 1             # <<<<<<<<<<<<<<
 *         unsigned int memory_size
 *         void *ptr
 */
  __pyx_v_adjusted_stripe_length = ((__pyx_v_columns + (2 * __pyx_v_radius)) + 1);

  /* "centrosome/_filter.pyx":186
 *     memory_size = (adjusted_stripe_length *
 *                    (sizeof(Histogram) + sizeof(PixelCount))+
 *                    sizeof(Histograms)+32)             # <<<<<<<<<<<<<<
 *     ptr = malloc(memory_size)
 *     memset(ptr, 0, memory_size)
 */
  __pyx_v_memory_size = (((__pyx_v_adjusted_stripe_length * ((sizeof(struct __pyx_t_10centrosome_7_filter_Histogram)) + (sizeof(struct __pyx_t_10centrosome_7_filter_PixelCount)))) + (sizeof(struct __pyx_t_10centrosome_7_filter_Histograms))) + 32);

  /* "centrosome/_filter.pyx":187
 *                    (sizeof(Histogram) + sizeof(PixelCount))+
 *                    sizeof(Histograms)+32)
 *     ptr = malloc(memory_size)             # <<<<<<<<<<<<<<
 *     memset(ptr, 0, memory_size)
 *     ph  = <Histograms *>ptr
 */
  __pyx_v_ptr = malloc(__pyx_v_memory_size);

  /* "centrosome/_filter.pyx":188
 *                    sizeof(Histograms)+32)
 *     ptr = malloc(memory_size)
 *     memset(ptr, 0, memory_size)             # <<<<<<<<<<<<<<
 *     ph  = <Histograms *>ptr
 *     if not ptr:
 */
  memset(__pyx_v_ptr, 0, __pyx_v_memory_size);

  /* "centrosome/_filter.pyx":189
 *     ptr = malloc(memory_size)
 *     memset(ptr, 0, memory_size)
 *     ph  = <Histograms *>ptr             # <<<<<<<<<<<<<<
 *     if not ptr:
 *         return ph
 */
  __pyx_v_ph = ((struct __pyx_t_10centrosome_7_filter_Histograms *)__pyx_v_ptr);

  /* "centrosome/_filter.pyx":190
 *     memset(ptr, 0, memory_size)
 *     ph  = <Histograms *>ptr
 *     if not ptr:             # <<<<<<<<<<<<<<
 *         return ph
 *     ph.memory = ptr
 */
  __pyx_t_1 = ((!(__pyx_v_ptr != 0)) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":191
 *     ph  = <Histograms *>ptr
 *     if not ptr:
 *         return ph             # <<<<<<<<<<<<<<
 *     ph.memory = ptr
 *     ptr = <void *>(ph+1)
 */
    __pyx_r = __pyx_v_ph;
    goto __pyx_L0;

    /* "centrosome/_filter.pyx":190
 *     memset(ptr, 0, memory_size)
 *     ph  = <Histograms *>ptr
 *     if not ptr:             # <<<<<<<<<<<<<<
 *         return ph
 *     ph.memory = ptr
 */
  }

  /* "centrosome/_filter.pyx":192
 *     if not ptr:
 *         return ph
 *     ph.memory = ptr             # <<<<<<<<<<<<<<
 *     ptr = <void *>(ph+1)
 *     ph.pixel_count = <PixelCount *>ptr
 */
  __pyx_v_ph->memory = __pyx_v_ptr;

  /* "centrosome/_filter.pyx":193
 *         return ph
 *     ph.memory = ptr
 *     ptr = <void *>(ph+1)             # <<<<<<<<<<<<<<
 *     ph.pixel_count = <PixelCount *>ptr
 *     ptr = <void *>(ph.pixel_count + adjusted_stripe_length)
 */
  __pyx_v_ptr = ((void *)(__pyx_v_ph + 1));

  /* "centrosome/_filter.pyx":194
 *     ph.memory = ptr
 *     ptr = <void *>(ph+1)
 *     ph.pixel_count = <PixelCount *>ptr             # <<<<<<<<<<<<<<
 *     ptr = <void *>(ph.pixel_count + adjusted_stripe_length)
 *     #
 */
  __pyx_v_ph->pixel_count = ((struct __pyx_t_10centrosome_7_filter_PixelCount *)__pyx_v_ptr);

  /* "centrosome/_filter.pyx":195
 *     ptr = <void *>(ph+1)
 *     ph.pixel_count = <PixelCount *>ptr
 *     ptr = <void *>(ph.pixel_count + adjusted_stripe_length)             # <<<<<<<<<<<<<<
 *     #
 *     # Align histogram memory to a 32-byte boundary
 */
  __pyx_v_ptr = ((void *)(__pyx_v_ph->pixel_count + __pyx_v_adjusted_stripe_length));

  /* "centrosome/_filter.pyx":199
 *     # Align histogram memory to a 32-byte boundary
 *     #
 *     roundoff     = <size_t>ptr             # <<<<<<<<<<<<<<
 *     roundoff    += 31
 *     roundoff    -= roundoff % 32
 */
  __pyx_v_roundoff = ((size_t)__pyx_v_ptr);

  /* "centrosome/_filter.pyx":200
 *     #
 *     roundoff     = <size_t>ptr
 *     roundoff    += 31             # <<<<<<<<<<<<<<
 *     roundoff    -= roundoff % 32
 *     ptr          = <void *>roundoff
 */
  __pyx_v_roundoff = (__pyx_v_roundoff + 31);

  /* "centrosome/_filter.pyx":201
 *     roundoff     = <size_t>ptr
 *     roundoff    += 31
 *     roundoff    -= roundoff % 32             # <<<<<<<<<<<<<<
 *     ptr          = <void *>roundoff
 *     ph.histogram = <Histogram *>ptr
 */
  __pyx_v_roundoff = (__pyx_v_roundoff - (__pyx_v_roundoff % 32));

  /* "centrosome/_filter.pyx":202
 *     roundoff    += 31
 *     roundoff    -= roundoff % 32
 *     ptr          = <void *>roundoff             # <<<<<<<<<<<<<<
 *     ph.histogram = <Histogram *>ptr
 *     #
 */
  __pyx_v_ptr = ((void *)__pyx_v_roundoff);

  /* "centrosome/_filter.pyx":203
 *     roundoff    -= roundoff % 32
 *     ptr          = <void *>roundoff
 *     ph.histogram = <Histogram *>ptr             # <<<<<<<<<<<<<<
 *     #
 *     # Fill in the statistical things we keep around
 */
  __pyx_v_ph->histogram = ((struct __pyx_t_10centrosome_7_filter_Histogram *)__pyx_v_ptr);

  /* "centrosome/_filter.pyx":207
 *     # Fill in the statistical things we keep around
 *     #
 *     ph.column_count   = columns             # <<<<<<<<<<<<<<
 *     ph.row_count      = rows
 *     ph.current_column = -radius
 */
  __pyx_v_ph->column_count = __pyx_v_columns;

  /* "centrosome/_filter.pyx":208
 *     #
 *     ph.column_count   = columns
 *     ph.row_count      = rows             # <<<<<<<<<<<<<<
 *     ph.current_column = -radius
 *     ph.stripe_length  = adjusted_stripe_length
 */
  __pyx_v_ph->row_count = __pyx_v_rows;

  /* "centrosome/_filter.pyx":209
 *     ph.column_count   = columns
 *     ph.row_count      = rows
 *     ph.current_column = -radius             # <<<<<<<<<<<<<<
 *     ph.stripe_length  = adjusted_stripe_length
 *     ph.current_row    = 0
 */
  __pyx_v_ph->current_column = (-__pyx_v_radius);

  /* "centrosome/_filter.pyx":210
 *     ph.row_count      = rows
 *     ph.current_column = -radius
 *     ph.stripe_length  = adjusted_stripe_length             # <<<<<<<<<<<<<<
 *     ph.current_row    = 0
 *     ph.radius         = radius
 */
  __pyx_v_ph->stripe_length = __pyx_v_adjusted_stripe_length;

  /* "centrosome/_filter.pyx":211
 *     ph.current_column = -radius
 *     ph.stripe_length  = adjusted_stripe_length
 *     ph.current_row    = 0             # <<<<<<<<<<<<<<
 *     ph.radius         = radius
 *     ph.percent        = percent
 */
  __pyx_v_ph->current_row = 0;

  /* "centrosome/_filter.pyx":212
 *     ph.stripe_length  = adjusted_stripe_length
 *     ph.current_row    = 0
 *     ph.radius         = radius             # <<<<<<<<<<<<<<
 *     ph.percent        = percent
 *     ph.row_stride     = row_stride
 */
  __pyx_v_ph->radius = __pyx_v_radius;

  /* "centrosome/_filter.pyx":213
 *     ph.current_row    = 0
 *     ph.radius         = radius
 *     ph.percent        = percent             # <<<<<<<<<<<<<<
 *     ph.row_stride     = row_stride
 *     ph.col_stride     = col_stride
 */
  __pyx_v_ph->percent = __pyx_v_percent;

  /* "centrosome/_filter.pyx":214
 *     ph.radius         = radius
 *     ph.percent        = percent
 *     ph.row_stride     = row_stride             # <<<<<<<<<<<<<<
 *     ph.col_stride     = col_stride
 *     ph.data           = data
 */
  __pyx_v_ph->row_stride = __pyx_v_row_stride;

  /* "centrosome/_filter.pyx":215
 *     ph.percent        = percent
 *     ph.row_stride     = row_stride
 *     ph.col_stride     = col_stride             # <<<<<<<<<<<<<<
 *     ph.data           = data
 *     ph.mask           = mask
 */
  __pyx_v_ph->col_stride = __pyx_v_col_stride;

  /* "centrosome/_filter.pyx":216
 *     ph.row_stride     = row_stride
 *     ph.col_stride     = col_stride
 *     ph.data           = data             # <<<<<<<<<<<<<<
 *     ph.mask           = mask
 *     ph.output         = output
 */
  __pyx_v_ph->data = __pyx_v_data;

  /* "centrosome/_filter.pyx":217
 *     ph.col_stride     = col_stride
 *     ph.data           = data
 *     ph.mask           = mask             # <<<<<<<<<<<<<<
 *     ph.output         = output
 *     #
 */
  __pyx_v_ph->mask = __pyx_v_mask;

  /* "centrosome/_filter.pyx":218
 *     ph.data           = data
 *     ph.mask           = mask
 *     ph.output         = output             # <<<<<<<<<<<<<<
 *     #
 *     # Compute the coordinates of the significant points
 */
  __pyx_v_ph->output = __pyx_v_output;

  /* "centrosome/_filter.pyx":232
 *     # corners
 *     #
 *     a = <int>(<np.float64_t>radius * 2.0 / 2.414213)             # <<<<<<<<<<<<<<
 *     a_2 = a / 2
 *     if a_2 == 0:
 */
  __pyx_v_a = ((int)((((__pyx_t_5numpy_float64_t)__pyx_v_radius) * 2.0) / 2.414213));

  /* "centrosome/_filter.pyx":233
 *     #
 *     a = <int>(<np.float64_t>radius * 2.0 / 2.414213)
 *     a_2 = a / 2             # <<<<<<<<<<<<<<
 *     if a_2 == 0:
 *         a_2 = 1
 */
  __pyx_t_2 = __Pyx_PyInt_From_long(__Pyx_div_long(__pyx_v_a, 2)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_a_2 = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "centrosome/_filter.pyx":234
 *     a = <int>(<np.float64_t>radius * 2.0 / 2.414213)
 *     a_2 = a / 2
 *     if a_2 == 0:             # <<<<<<<<<<<<<<
 *         a_2 = 1
 *     ph.a_2 = a_2
 */
  __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_a_2, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 234; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 234; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":235
 *     a_2 = a / 2
 *     if a_2 == 0:
 *         a_2 = 1             # <<<<<<<<<<<<<<
 *     ph.a_2 = a_2
 *     if radius <= a_2:
 */
    __Pyx_INCREF(__pyx_int_1);
    __Pyx_DECREF_SET(__pyx_v_a_2, __pyx_int_1);

    /* "centrosome/_filter.pyx":234
 *     a = <int>(<np.float64_t>radius * 2.0 / 2.414213)
 *     a_2 = a / 2
 *     if a_2 == 0:             # <<<<<<<<<<<<<<
 *         a_2 = 1
 *     ph.a_2 = a_2
 */
  }

  /* "centrosome/_filter.pyx":236
 *     if a_2 == 0:
 *         a_2 = 1
 *     ph.a_2 = a_2             # <<<<<<<<<<<<<<
 *     if radius <= a_2:
 *         radius = a_2+1
 */
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_v_a_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 236; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ph->a_2 = __pyx_t_3;

  /* "centrosome/_filter.pyx":237
 *         a_2 = 1
 *     ph.a_2 = a_2
 *     if radius <= a_2:             # <<<<<<<<<<<<<<
 *         radius = a_2+1
 *         ph.radius = radius
 */
  __pyx_t_2 = __Pyx_PyInt_From_npy_int32(__pyx_v_radius); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_v_a_2, Py_LE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 237; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":238
 *     ph.a_2 = a_2
 *     if radius <= a_2:
 *         radius = a_2+1             # <<<<<<<<<<<<<<
 *         ph.radius = radius
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_a_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 238; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_4); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 238; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_radius = __pyx_t_3;

    /* "centrosome/_filter.pyx":239
 *     if radius <= a_2:
 *         radius = a_2+1
 *         ph.radius = radius             # <<<<<<<<<<<<<<
 * 
 *     ph.last_top_left.x = -a_2
 */
    __pyx_v_ph->radius = __pyx_v_radius;

    /* "centrosome/_filter.pyx":237
 *         a_2 = 1
 *     ph.a_2 = a_2
 *     if radius <= a_2:             # <<<<<<<<<<<<<<
 *         radius = a_2+1
 *         ph.radius = radius
 */
  }

  /* "centrosome/_filter.pyx":241
 *         ph.radius = radius
 * 
 *     ph.last_top_left.x = -a_2             # <<<<<<<<<<<<<<
 *     ph.last_top_left.y = -radius - 1
 * 
 */
  __pyx_t_4 = PyNumber_Negative(__pyx_v_a_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_4); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ph->last_top_left.x = __pyx_t_3;

  /* "centrosome/_filter.pyx":242
 * 
 *     ph.last_top_left.x = -a_2
 *     ph.last_top_left.y = -radius - 1             # <<<<<<<<<<<<<<
 * 
 *     ph.top_left.x = -radius
 */
  __pyx_v_ph->last_top_left.y = ((-__pyx_v_radius) - 1);

  /* "centrosome/_filter.pyx":244
 *     ph.last_top_left.y = -radius - 1
 * 
 *     ph.top_left.x = -radius             # <<<<<<<<<<<<<<
 *     ph.top_left.y = -a_2 - 1
 * 
 */
  __pyx_v_ph->top_left.x = (-__pyx_v_radius);

  /* "centrosome/_filter.pyx":245
 * 
 *     ph.top_left.x = -radius
 *     ph.top_left.y = -a_2 - 1             # <<<<<<<<<<<<<<
 * 
 *     ph.last_top_right.x = a_2 - 1
 */
  __pyx_t_4 = PyNumber_Negative(__pyx_v_a_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_SubtractObjC(__pyx_t_4, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ph->top_left.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":247
 *     ph.top_left.y = -a_2 - 1
 * 
 *     ph.last_top_right.x = a_2 - 1             # <<<<<<<<<<<<<<
 *     ph.last_top_right.y = -radius - 1
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_SubtractObjC(__pyx_v_a_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ph->last_top_right.x = __pyx_t_3;

  /* "centrosome/_filter.pyx":248
 * 
 *     ph.last_top_right.x = a_2 - 1
 *     ph.last_top_right.y = -radius - 1             # <<<<<<<<<<<<<<
 * 
 *     ph.top_right.x = radius - 1
 */
  __pyx_v_ph->last_top_right.y = ((-__pyx_v_radius) - 1);

  /* "centrosome/_filter.pyx":250
 *     ph.last_top_right.y = -radius - 1
 * 
 *     ph.top_right.x = radius - 1             # <<<<<<<<<<<<<<
 *     ph.top_right.y = -a_2 - 1
 * 
 */
  __pyx_v_ph->top_right.x = (__pyx_v_radius - 1);

  /* "centrosome/_filter.pyx":251
 * 
 *     ph.top_right.x = radius - 1
 *     ph.top_right.y = -a_2 - 1             # <<<<<<<<<<<<<<
 * 
 *     ph.last_leading_edge.x = radius
 */
  __pyx_t_2 = PyNumber_Negative(__pyx_v_a_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_t_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_4); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ph->top_right.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":253
 *     ph.top_right.y = -a_2 - 1
 * 
 *     ph.last_leading_edge.x = radius             # <<<<<<<<<<<<<<
 *     ph.last_leading_edge.y = -a_2 - 1
 * 
 */
  __pyx_v_ph->last_leading_edge.x = __pyx_v_radius;

  /* "centrosome/_filter.pyx":254
 * 
 *     ph.last_leading_edge.x = radius
 *     ph.last_leading_edge.y = -a_2 - 1             # <<<<<<<<<<<<<<
 * 
 *     ph.leading_edge.x = radius
 */
  __pyx_t_4 = PyNumber_Negative(__pyx_v_a_2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_SubtractObjC(__pyx_t_4, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ph->last_leading_edge.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":256
 *     ph.last_leading_edge.y = -a_2 - 1
 * 
 *     ph.leading_edge.x = radius             # <<<<<<<<<<<<<<
 *     ph.leading_edge.y = a_2
 * 
 */
  __pyx_v_ph->leading_edge.x = __pyx_v_radius;

  /* "centrosome/_filter.pyx":257
 * 
 *     ph.leading_edge.x = radius
 *     ph.leading_edge.y = a_2             # <<<<<<<<<<<<<<
 * 
 *     ph.last_bottom_right.x = radius
 */
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_v_a_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ph->leading_edge.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":259
 *     ph.leading_edge.y = a_2
 * 
 *     ph.last_bottom_right.x = radius             # <<<<<<<<<<<<<<
 *     ph.last_bottom_right.y = a_2
 * 
 */
  __pyx_v_ph->last_bottom_right.x = __pyx_v_radius;

  /* "centrosome/_filter.pyx":260
 * 
 *     ph.last_bottom_right.x = radius
 *     ph.last_bottom_right.y = a_2             # <<<<<<<<<<<<<<
 * 
 *     ph.bottom_right.x = a_2
 */
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_v_a_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ph->last_bottom_right.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":262
 *     ph.last_bottom_right.y = a_2
 * 
 *     ph.bottom_right.x = a_2             # <<<<<<<<<<<<<<
 *     ph.bottom_right.y = radius
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_v_a_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 262; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ph->bottom_right.x = __pyx_t_3;

  /* "centrosome/_filter.pyx":263
 * 
 *     ph.bottom_right.x = a_2
 *     ph.bottom_right.y = radius             # <<<<<<<<<<<<<<
 * 
 *     ph.last_bottom_left.x = -radius-1
 */
  __pyx_v_ph->bottom_right.y = __pyx_v_radius;

  /* "centrosome/_filter.pyx":265
 *     ph.bottom_right.y = radius
 * 
 *     ph.last_bottom_left.x = -radius-1             # <<<<<<<<<<<<<<
 *     ph.last_bottom_left.y = a_2
 * 
 */
  __pyx_v_ph->last_bottom_left.x = ((-__pyx_v_radius) - 1);

  /* "centrosome/_filter.pyx":266
 * 
 *     ph.last_bottom_left.x = -radius-1
 *     ph.last_bottom_left.y = a_2             # <<<<<<<<<<<<<<
 * 
 *     ph.bottom_left.x = -a_2-1
 */
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_v_a_2); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_ph->last_bottom_left.y = __pyx_t_3;

  /* "centrosome/_filter.pyx":268
 *     ph.last_bottom_left.y = a_2
 * 
 *     ph.bottom_left.x = -a_2-1             # <<<<<<<<<<<<<<
 *     ph.bottom_left.y = radius
 * 
 */
  __pyx_t_2 = PyNumber_Negative(__pyx_v_a_2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_t_2, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_npy_int32(__pyx_t_4); if (unlikely((__pyx_t_3 == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ph->bottom_left.x = __pyx_t_3;

  /* "centrosome/_filter.pyx":269
 * 
 *     ph.bottom_left.x = -a_2-1
 *     ph.bottom_left.y = radius             # <<<<<<<<<<<<<<
 * 
 *     #
 */
  __pyx_v_ph->bottom_left.y = __pyx_v_radius;

  /* "centrosome/_filter.pyx":274
 *     # Set the stride of each SCoord based on its x and y
 *     #
 *     set_stride(ph, &ph.last_top_left)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.top_left)
 *     set_stride(ph, &ph.last_top_right)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->last_top_left));

  /* "centrosome/_filter.pyx":275
 *     #
 *     set_stride(ph, &ph.last_top_left)
 *     set_stride(ph, &ph.top_left)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.last_top_right)
 *     set_stride(ph, &ph.top_right)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->top_left));

  /* "centrosome/_filter.pyx":276
 *     set_stride(ph, &ph.last_top_left)
 *     set_stride(ph, &ph.top_left)
 *     set_stride(ph, &ph.last_top_right)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.top_right)
 *     set_stride(ph, &ph.last_leading_edge)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->last_top_right));

  /* "centrosome/_filter.pyx":277
 *     set_stride(ph, &ph.top_left)
 *     set_stride(ph, &ph.last_top_right)
 *     set_stride(ph, &ph.top_right)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.last_leading_edge)
 *     set_stride(ph, &ph.leading_edge)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->top_right));

  /* "centrosome/_filter.pyx":278
 *     set_stride(ph, &ph.last_top_right)
 *     set_stride(ph, &ph.top_right)
 *     set_stride(ph, &ph.last_leading_edge)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.leading_edge)
 *     set_stride(ph, &ph.last_bottom_left)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->last_leading_edge));

  /* "centrosome/_filter.pyx":279
 *     set_stride(ph, &ph.top_right)
 *     set_stride(ph, &ph.last_leading_edge)
 *     set_stride(ph, &ph.leading_edge)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.last_bottom_left)
 *     set_stride(ph, &ph.bottom_left)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->leading_edge));

  /* "centrosome/_filter.pyx":280
 *     set_stride(ph, &ph.last_leading_edge)
 *     set_stride(ph, &ph.leading_edge)
 *     set_stride(ph, &ph.last_bottom_left)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.bottom_left)
 *     set_stride(ph, &ph.last_bottom_right)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->last_bottom_left));

  /* "centrosome/_filter.pyx":281
 *     set_stride(ph, &ph.leading_edge)
 *     set_stride(ph, &ph.last_bottom_left)
 *     set_stride(ph, &ph.bottom_left)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.last_bottom_right)
 *     set_stride(ph, &ph.bottom_right)
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->bottom_left));

  /* "centrosome/_filter.pyx":282
 *     set_stride(ph, &ph.last_bottom_left)
 *     set_stride(ph, &ph.bottom_left)
 *     set_stride(ph, &ph.last_bottom_right)             # <<<<<<<<<<<<<<
 *     set_stride(ph, &ph.bottom_right)
 * 
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->last_bottom_right));

  /* "centrosome/_filter.pyx":283
 *     set_stride(ph, &ph.bottom_left)
 *     set_stride(ph, &ph.last_bottom_right)
 *     set_stride(ph, &ph.bottom_right)             # <<<<<<<<<<<<<<
 * 
 *     return ph
 */
  __pyx_f_10centrosome_7_filter_set_stride(__pyx_v_ph, (&__pyx_v_ph->bottom_right));

  /* "centrosome/_filter.pyx":285
 *     set_stride(ph, &ph.bottom_right)
 * 
 *     return ph             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_r = __pyx_v_ph;
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":166
 * #
 * ############################################################################
 * cdef Histograms *allocate_histograms(np.int32_t   rows,             # <<<<<<<<<<<<<<
 *                                             np.int32_t   columns,
 *                                             np.int32_t   row_stride,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("centrosome._filter.allocate_histograms", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_a_2);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":292
 * #
 * ############################################################################
 * cdef void free_histograms(Histograms *ph):             # <<<<<<<<<<<<<<
 *     free(ph.memory)
 * 
 */

static void __pyx_f_10centrosome_7_filter_free_histograms(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("free_histograms", 0);

  /* "centrosome/_filter.pyx":293
 * ############################################################################
 * cdef void free_histograms(Histograms *ph):
 *     free(ph.memory)             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  free(__pyx_v_ph->memory);

  /* "centrosome/_filter.pyx":292
 * #
 * ############################################################################
 * cdef void free_histograms(Histograms *ph):             # <<<<<<<<<<<<<<
 *     free(ph.memory)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":301
 * ############################################################################
 * 
 * cdef void set_stride(Histograms *ph, SCoord *psc):             # <<<<<<<<<<<<<<
 *     psc.stride = psc.x * ph.col_stride + psc.y * ph.row_stride
 * 
 */

static void __pyx_f_10centrosome_7_filter_set_stride(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, struct __pyx_t_10centrosome_7_filter_SCoord *__pyx_v_psc) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_stride", 0);

  /* "centrosome/_filter.pyx":302
 * 
 * cdef void set_stride(Histograms *ph, SCoord *psc):
 *     psc.stride = psc.x * ph.col_stride + psc.y * ph.row_stride             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_v_psc->stride = ((__pyx_v_psc->x * __pyx_v_ph->col_stride) + (__pyx_v_psc->y * __pyx_v_ph->row_stride));

  /* "centrosome/_filter.pyx":301
 * ############################################################################
 * 
 * cdef void set_stride(Histograms *ph, SCoord *psc):             # <<<<<<<<<<<<<<
 *     psc.stride = psc.x * ph.col_stride + psc.y * ph.row_stride
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":326
 * #
 * ############################################################################
 * cdef inline np.int32_t tl_br_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius + ph.current_row)%ph.stripe_length
 * 
 */

static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_tl_br_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  __pyx_t_5numpy_int32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tl_br_colidx", 0);

  /* "centrosome/_filter.pyx":327
 * ############################################################################
 * cdef inline np.int32_t tl_br_colidx(Histograms *ph, np.int32_t colidx):
 *     return (colidx + 3*ph.radius + ph.current_row)%ph.stripe_length             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.int32_t tr_bl_colidx(Histograms *ph, np.int32_t colidx):
 */
  __pyx_t_1 = ((__pyx_v_colidx + (3 * __pyx_v_ph->radius)) + __pyx_v_ph->current_row);
  if (unlikely(__pyx_v_ph->stripe_length == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 327; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __Pyx_mod_long(__pyx_t_1, __pyx_v_ph->stripe_length);
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":326
 * #
 * ############################################################################
 * cdef inline np.int32_t tl_br_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius + ph.current_row)%ph.stripe_length
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("centrosome._filter.tl_br_colidx", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":329
 *     return (colidx + 3*ph.radius + ph.current_row)%ph.stripe_length
 * 
 * cdef inline np.int32_t tr_bl_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius + ph.row_count-ph.current_row) % ph.stripe_length
 * 
 */

static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_tr_bl_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  __pyx_t_5numpy_int32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("tr_bl_colidx", 0);

  /* "centrosome/_filter.pyx":330
 * 
 * cdef inline np.int32_t tr_bl_colidx(Histograms *ph, np.int32_t colidx):
 *     return (colidx + 3*ph.radius + ph.row_count-ph.current_row) % ph.stripe_length             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.int32_t leading_edge_colidx(Histograms *ph, np.int32_t colidx):
 */
  __pyx_t_1 = (((__pyx_v_colidx + (3 * __pyx_v_ph->radius)) + __pyx_v_ph->row_count) - __pyx_v_ph->current_row);
  if (unlikely(__pyx_v_ph->stripe_length == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __Pyx_mod_long(__pyx_t_1, __pyx_v_ph->stripe_length);
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":329
 *     return (colidx + 3*ph.radius + ph.current_row)%ph.stripe_length
 * 
 * cdef inline np.int32_t tr_bl_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius + ph.row_count-ph.current_row) % ph.stripe_length
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("centrosome._filter.tr_bl_colidx", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":332
 *     return (colidx + 3*ph.radius + ph.row_count-ph.current_row) % ph.stripe_length
 * 
 * cdef inline np.int32_t leading_edge_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 5*ph.radius) % ph.stripe_length
 * 
 */

static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_leading_edge_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  __pyx_t_5numpy_int32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("leading_edge_colidx", 0);

  /* "centrosome/_filter.pyx":333
 * 
 * cdef inline np.int32_t leading_edge_colidx(Histograms *ph, np.int32_t colidx):
 *     return (colidx + 5*ph.radius) % ph.stripe_length             # <<<<<<<<<<<<<<
 * 
 * cdef inline np.int32_t trailing_edge_colidx(Histograms *ph, np.int32_t colidx):
 */
  __pyx_t_1 = (__pyx_v_colidx + (5 * __pyx_v_ph->radius));
  if (unlikely(__pyx_v_ph->stripe_length == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __Pyx_mod_long(__pyx_t_1, __pyx_v_ph->stripe_length);
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":332
 *     return (colidx + 3*ph.radius + ph.row_count-ph.current_row) % ph.stripe_length
 * 
 * cdef inline np.int32_t leading_edge_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 5*ph.radius) % ph.stripe_length
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("centrosome._filter.leading_edge_colidx", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":335
 *     return (colidx + 5*ph.radius) % ph.stripe_length
 * 
 * cdef inline np.int32_t trailing_edge_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius - 1) % ph.stripe_length
 * #
 */

static CYTHON_INLINE __pyx_t_5numpy_int32_t __pyx_f_10centrosome_7_filter_trailing_edge_colidx(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  __pyx_t_5numpy_int32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("trailing_edge_colidx", 0);

  /* "centrosome/_filter.pyx":336
 * 
 * cdef inline np.int32_t trailing_edge_colidx(Histograms *ph, np.int32_t colidx):
 *     return (colidx + 3*ph.radius - 1) % ph.stripe_length             # <<<<<<<<<<<<<<
 * #
 * # add16 - add 16 consecutive integers
 */
  __pyx_t_1 = ((__pyx_v_colidx + (3 * __pyx_v_ph->radius)) - 1);
  if (unlikely(__pyx_v_ph->stripe_length == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __Pyx_mod_long(__pyx_t_1, __pyx_v_ph->stripe_length);
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":335
 *     return (colidx + 5*ph.radius) % ph.stripe_length
 * 
 * cdef inline np.int32_t trailing_edge_colidx(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     return (colidx + 3*ph.radius - 1) % ph.stripe_length
 * #
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("centrosome._filter.trailing_edge_colidx", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":344
 * # TO_DO - optimize using SIMD instructions
 * #
 * cdef inline void add16(np.uint16_t *dest, np.uint16_t *src):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(16):
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_add16(__pyx_t_5numpy_uint16_t *__pyx_v_dest, __pyx_t_5numpy_uint16_t *__pyx_v_src) {
  int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("add16", 0);

  /* "centrosome/_filter.pyx":346
 * cdef inline void add16(np.uint16_t *dest, np.uint16_t *src):
 *     cdef int i
 *     for i in range(16):             # <<<<<<<<<<<<<<
 *         dest[i] += src[i]
 * 
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 16; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "centrosome/_filter.pyx":347
 *     cdef int i
 *     for i in range(16):
 *         dest[i] += src[i]             # <<<<<<<<<<<<<<
 * 
 * cdef inline void sub16(np.uint16_t *dest, np.uint16_t *src):
 */
    __pyx_t_2 = __pyx_v_i;
    (__pyx_v_dest[__pyx_t_2]) = ((__pyx_v_dest[__pyx_t_2]) + (__pyx_v_src[__pyx_v_i]));
  }

  /* "centrosome/_filter.pyx":344
 * # TO_DO - optimize using SIMD instructions
 * #
 * cdef inline void add16(np.uint16_t *dest, np.uint16_t *src):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(16):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":349
 *         dest[i] += src[i]
 * 
 * cdef inline void sub16(np.uint16_t *dest, np.uint16_t *src):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(16):
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_sub16(__pyx_t_5numpy_uint16_t *__pyx_v_dest, __pyx_t_5numpy_uint16_t *__pyx_v_src) {
  int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("sub16", 0);

  /* "centrosome/_filter.pyx":351
 * cdef inline void sub16(np.uint16_t *dest, np.uint16_t *src):
 *     cdef int i
 *     for i in range(16):             # <<<<<<<<<<<<<<
 *         dest[i] -= src[i]
 * 
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 16; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "centrosome/_filter.pyx":352
 *     cdef int i
 *     for i in range(16):
 *         dest[i] -= src[i]             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
    __pyx_t_2 = __pyx_v_i;
    (__pyx_v_dest[__pyx_t_2]) = ((__pyx_v_dest[__pyx_t_2]) - (__pyx_v_src[__pyx_v_i]));
  }

  /* "centrosome/_filter.pyx":349
 *         dest[i] += src[i]
 * 
 * cdef inline void sub16(np.uint16_t *dest, np.uint16_t *src):             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i in range(16):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":363
 * #
 * ############################################################################
 * cdef inline void accumulate_coarse_histogram(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int offset
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate_coarse_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  int __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("accumulate_coarse_histogram", 0);

  /* "centrosome/_filter.pyx":367
 *         int offset
 * 
 *     offset = tr_bl_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     if ph.pixel_count[offset].top_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":368
 * 
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_right > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 */
  __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).top_right > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":369
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)             # <<<<<<<<<<<<<<
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 *     offset = leading_edge_colidx(ph, colidx)
 */
    __pyx_f_10centrosome_7_filter_add16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).top_right.coarse);

    /* "centrosome/_filter.pyx":370
 *     if ph.pixel_count[offset].top_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].top_right             # <<<<<<<<<<<<<<
 *     offset = leading_edge_colidx(ph, colidx)
 *     if ph.pixel_count[offset].edge > 0:
 */
    __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count + (__pyx_v_ph->pixel_count[__pyx_v_offset]).top_right);

    /* "centrosome/_filter.pyx":368
 * 
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_right > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 */
  }

  /* "centrosome/_filter.pyx":371
 *         add16(ph.accumulator.coarse, ph.histogram[offset].top_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 *     offset = leading_edge_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     if ph.pixel_count[offset].edge > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_leading_edge_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":372
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 *     offset = leading_edge_colidx(ph, colidx)
 *     if ph.pixel_count[offset].edge > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 */
  __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).edge > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":373
 *     offset = leading_edge_colidx(ph, colidx)
 *     if ph.pixel_count[offset].edge > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)             # <<<<<<<<<<<<<<
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 *     offset = tl_br_colidx(ph, colidx)
 */
    __pyx_f_10centrosome_7_filter_add16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).edge.coarse);

    /* "centrosome/_filter.pyx":374
 *     if ph.pixel_count[offset].edge > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].edge             # <<<<<<<<<<<<<<
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_right > 0:
 */
    __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count + (__pyx_v_ph->pixel_count[__pyx_v_offset]).edge);

    /* "centrosome/_filter.pyx":372
 *         ph.accumulator_count += ph.pixel_count[offset].top_right
 *     offset = leading_edge_colidx(ph, colidx)
 *     if ph.pixel_count[offset].edge > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 */
  }

  /* "centrosome/_filter.pyx":375
 *         add16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 *     offset = tl_br_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     if ph.pixel_count[offset].bottom_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].bottom_right.coarse)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":376
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_right > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].bottom_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].bottom_right
 */
  __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).bottom_right > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":377
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].bottom_right.coarse)             # <<<<<<<<<<<<<<
 *         ph.accumulator_count += ph.pixel_count[offset].bottom_right
 * 
 */
    __pyx_f_10centrosome_7_filter_add16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).bottom_right.coarse);

    /* "centrosome/_filter.pyx":378
 *     if ph.pixel_count[offset].bottom_right > 0:
 *         add16(ph.accumulator.coarse, ph.histogram[offset].bottom_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].bottom_right             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
    __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count + (__pyx_v_ph->pixel_count[__pyx_v_offset]).bottom_right);

    /* "centrosome/_filter.pyx":376
 *         ph.accumulator_count += ph.pixel_count[offset].edge
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_right > 0:             # <<<<<<<<<<<<<<
 *         add16(ph.accumulator.coarse, ph.histogram[offset].bottom_right.coarse)
 *         ph.accumulator_count += ph.pixel_count[offset].bottom_right
 */
  }

  /* "centrosome/_filter.pyx":363
 * #
 * ############################################################################
 * cdef inline void accumulate_coarse_histogram(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int offset
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":386
 * #
 * ############################################################################
 * cdef inline void deaccumulate_coarse_histogram(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int offset
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_deaccumulate_coarse_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx) {
  int __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("deaccumulate_coarse_histogram", 0);

  /* "centrosome/_filter.pyx":392
 *     # The trailing diagonals don't appear until here
 *     #
 *     if colidx <= ph.a_2:             # <<<<<<<<<<<<<<
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 */
  __pyx_t_1 = ((__pyx_v_colidx <= __pyx_v_ph->a_2) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":393
 *     #
 *     if colidx <= ph.a_2:
 *         return             # <<<<<<<<<<<<<<
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_left > 0:
 */
    goto __pyx_L0;

    /* "centrosome/_filter.pyx":392
 *     # The trailing diagonals don't appear until here
 *     #
 *     if colidx <= ph.a_2:             # <<<<<<<<<<<<<<
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 */
  }

  /* "centrosome/_filter.pyx":394
 *     if colidx <= ph.a_2:
 *         return
 *     offset = tl_br_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     if ph.pixel_count[offset].top_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].top_left.coarse)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":395
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_left > 0:             # <<<<<<<<<<<<<<
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].top_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].top_left
 */
  __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).top_left > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":396
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].top_left.coarse)             # <<<<<<<<<<<<<<
 *         ph.accumulator_count -= ph.pixel_count[offset].top_left
 *     #
 */
    __pyx_f_10centrosome_7_filter_sub16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).top_left.coarse);

    /* "centrosome/_filter.pyx":397
 *     if ph.pixel_count[offset].top_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].top_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].top_left             # <<<<<<<<<<<<<<
 *     #
 *     # The trailing edge doesn't appear from the border until here
 */
    __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count - (__pyx_v_ph->pixel_count[__pyx_v_offset]).top_left);

    /* "centrosome/_filter.pyx":395
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 *     if ph.pixel_count[offset].top_left > 0:             # <<<<<<<<<<<<<<
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].top_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].top_left
 */
  }

  /* "centrosome/_filter.pyx":401
 *     # The trailing edge doesn't appear from the border until here
 *     #
 *     if colidx > ph.radius:             # <<<<<<<<<<<<<<
 *         offset = trailing_edge_colidx(ph, colidx)
 *         if ph.pixel_count[offset].edge > 0:
 */
  __pyx_t_1 = ((__pyx_v_colidx > __pyx_v_ph->radius) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":402
 *     #
 *     if colidx > ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *         if ph.pixel_count[offset].edge > 0:
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 */
    __pyx_v_offset = __pyx_f_10centrosome_7_filter_trailing_edge_colidx(__pyx_v_ph, __pyx_v_colidx);

    /* "centrosome/_filter.pyx":403
 *     if colidx > ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)
 *         if ph.pixel_count[offset].edge > 0:             # <<<<<<<<<<<<<<
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 */
    __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).edge > 0) != 0);
    if (__pyx_t_1) {

      /* "centrosome/_filter.pyx":404
 *         offset = trailing_edge_colidx(ph, colidx)
 *         if ph.pixel_count[offset].edge > 0:
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)             # <<<<<<<<<<<<<<
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 *     offset = tr_bl_colidx(ph, colidx)
 */
      __pyx_f_10centrosome_7_filter_sub16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).edge.coarse);

      /* "centrosome/_filter.pyx":405
 *         if ph.pixel_count[offset].edge > 0:
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *             ph.accumulator_count -= ph.pixel_count[offset].edge             # <<<<<<<<<<<<<<
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_left > 0:
 */
      __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count - (__pyx_v_ph->pixel_count[__pyx_v_offset]).edge);

      /* "centrosome/_filter.pyx":403
 *     if colidx > ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)
 *         if ph.pixel_count[offset].edge > 0:             # <<<<<<<<<<<<<<
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 */
    }

    /* "centrosome/_filter.pyx":401
 *     # The trailing edge doesn't appear from the border until here
 *     #
 *     if colidx > ph.radius:             # <<<<<<<<<<<<<<
 *         offset = trailing_edge_colidx(ph, colidx)
 *         if ph.pixel_count[offset].edge > 0:
 */
  }

  /* "centrosome/_filter.pyx":406
 *             sub16(ph.accumulator.coarse, ph.histogram[offset].edge.coarse)
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 *     offset = tr_bl_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     if ph.pixel_count[offset].bottom_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].bottom_left.coarse)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":407
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_left > 0:             # <<<<<<<<<<<<<<
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].bottom_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].bottom_left
 */
  __pyx_t_1 = (((__pyx_v_ph->pixel_count[__pyx_v_offset]).bottom_left > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":408
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].bottom_left.coarse)             # <<<<<<<<<<<<<<
 *         ph.accumulator_count -= ph.pixel_count[offset].bottom_left
 * 
 */
    __pyx_f_10centrosome_7_filter_sub16(__pyx_v_ph->accumulator.coarse, (__pyx_v_ph->histogram[__pyx_v_offset]).bottom_left.coarse);

    /* "centrosome/_filter.pyx":409
 *     if ph.pixel_count[offset].bottom_left > 0:
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].bottom_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].bottom_left             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
    __pyx_v_ph->accumulator_count = (__pyx_v_ph->accumulator_count - (__pyx_v_ph->pixel_count[__pyx_v_offset]).bottom_left);

    /* "centrosome/_filter.pyx":407
 *             ph.accumulator_count -= ph.pixel_count[offset].edge
 *     offset = tr_bl_colidx(ph, colidx)
 *     if ph.pixel_count[offset].bottom_left > 0:             # <<<<<<<<<<<<<<
 *         sub16(ph.accumulator.coarse, ph.histogram[offset].bottom_left.coarse)
 *         ph.accumulator_count -= ph.pixel_count[offset].bottom_left
 */
  }

  /* "centrosome/_filter.pyx":386
 * #
 * ############################################################################
 * cdef inline void deaccumulate_coarse_histogram(Histograms *ph, np.int32_t colidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int offset
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":416
 * #
 * ############################################################################
 * cdef inline void accumulate_fine_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                            np.int32_t colidx,
 *                                            np.uint32_t fineidx):
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate_fine_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx, __pyx_t_5numpy_uint32_t __pyx_v_fineidx) {
  int __pyx_v_fineoffset;
  int __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("accumulate_fine_histogram", 0);

  /* "centrosome/_filter.pyx":420
 *                                            np.uint32_t fineidx):
 *     cdef:
 *         int fineoffset = fineidx * 16             # <<<<<<<<<<<<<<
 *         int offset
 * 
 */
  __pyx_v_fineoffset = (__pyx_v_fineidx * 16);

  /* "centrosome/_filter.pyx":423
 *         int offset
 * 
 *     offset = tr_bl_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_right.fine+fineoffset)
 *     offset = leading_edge_colidx(ph, colidx)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":424
 * 
 *     offset = tr_bl_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_right.fine+fineoffset)             # <<<<<<<<<<<<<<
 *     offset = leading_edge_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 */
  __pyx_f_10centrosome_7_filter_add16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).top_right.fine + __pyx_v_fineoffset));

  /* "centrosome/_filter.pyx":425
 *     offset = tr_bl_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_right.fine+fineoffset)
 *     offset = leading_edge_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tl_br_colidx(ph, colidx)
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_leading_edge_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":426
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_right.fine+fineoffset)
 *     offset = leading_edge_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)             # <<<<<<<<<<<<<<
 *     offset = tl_br_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_right.fine+fineoffset)
 */
  __pyx_f_10centrosome_7_filter_add16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).edge.fine + __pyx_v_fineoffset));

  /* "centrosome/_filter.pyx":427
 *     offset = leading_edge_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tl_br_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_right.fine+fineoffset)
 * 
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":428
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tl_br_colidx(ph, colidx)
 *     add16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_right.fine+fineoffset)             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_f_10centrosome_7_filter_add16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).bottom_right.fine + __pyx_v_fineoffset));

  /* "centrosome/_filter.pyx":416
 * #
 * ############################################################################
 * cdef inline void accumulate_fine_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                            np.int32_t colidx,
 *                                            np.uint32_t fineidx):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":435
 * #
 * ############################################################################
 * cdef inline void deaccumulate_fine_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                              np.int32_t colidx,
 *                                              np.uint32_t fineidx):
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_deaccumulate_fine_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, __pyx_t_5numpy_int32_t __pyx_v_colidx, __pyx_t_5numpy_uint32_t __pyx_v_fineidx) {
  int __pyx_v_fineoffset;
  int __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("deaccumulate_fine_histogram", 0);

  /* "centrosome/_filter.pyx":439
 *                                              np.uint32_t fineidx):
 *     cdef:
 *         int fineoffset = fineidx * 16             # <<<<<<<<<<<<<<
 *         int offset
 * 
 */
  __pyx_v_fineoffset = (__pyx_v_fineidx * 16);

  /* "centrosome/_filter.pyx":445
 *     # The trailing diagonals don't appear until here
 *     #
 *     if colidx < ph.a_2:             # <<<<<<<<<<<<<<
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 */
  __pyx_t_1 = ((__pyx_v_colidx < __pyx_v_ph->a_2) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":446
 *     #
 *     if colidx < ph.a_2:
 *         return             # <<<<<<<<<<<<<<
 *     offset = tl_br_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)
 */
    goto __pyx_L0;

    /* "centrosome/_filter.pyx":445
 *     # The trailing diagonals don't appear until here
 *     #
 *     if colidx < ph.a_2:             # <<<<<<<<<<<<<<
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 */
  }

  /* "centrosome/_filter.pyx":447
 *     if colidx < ph.a_2:
 *         return
 *     offset = tl_br_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)
 *     if colidx >= ph.radius:
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":448
 *         return
 *     offset = tl_br_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)             # <<<<<<<<<<<<<<
 *     if colidx >= ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)
 */
  __pyx_f_10centrosome_7_filter_sub16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).top_left.fine + __pyx_v_fineoffset));

  /* "centrosome/_filter.pyx":449
 *     offset = tl_br_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)
 *     if colidx >= ph.radius:             # <<<<<<<<<<<<<<
 *         offset = trailing_edge_colidx(ph, colidx)
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 */
  __pyx_t_1 = ((__pyx_v_colidx >= __pyx_v_ph->radius) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":450
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)
 *     if colidx >= ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tr_bl_colidx(ph, colidx)
 */
    __pyx_v_offset = __pyx_f_10centrosome_7_filter_trailing_edge_colidx(__pyx_v_ph, __pyx_v_colidx);

    /* "centrosome/_filter.pyx":451
 *     if colidx >= ph.radius:
 *         offset = trailing_edge_colidx(ph, colidx)
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)             # <<<<<<<<<<<<<<
 *     offset = tr_bl_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_left.fine+fineoffset)
 */
    __pyx_f_10centrosome_7_filter_sub16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).edge.fine + __pyx_v_fineoffset));

    /* "centrosome/_filter.pyx":449
 *     offset = tl_br_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].top_left.fine+fineoffset)
 *     if colidx >= ph.radius:             # <<<<<<<<<<<<<<
 *         offset = trailing_edge_colidx(ph, colidx)
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 */
  }

  /* "centrosome/_filter.pyx":452
 *         offset = trailing_edge_colidx(ph, colidx)
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tr_bl_colidx(ph, colidx)             # <<<<<<<<<<<<<<
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_left.fine+fineoffset)
 * 
 */
  __pyx_v_offset = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_colidx);

  /* "centrosome/_filter.pyx":453
 *         sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].edge.fine+fineoffset)
 *     offset = tr_bl_colidx(ph, colidx)
 *     sub16(ph.accumulator.fine+fineoffset, ph.histogram[offset].bottom_left.fine+fineoffset)             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_f_10centrosome_7_filter_sub16((__pyx_v_ph->accumulator.fine + __pyx_v_fineoffset), ((__pyx_v_ph->histogram[__pyx_v_offset]).bottom_left.fine + __pyx_v_fineoffset));

  /* "centrosome/_filter.pyx":435
 * #
 * ############################################################################
 * cdef inline void deaccumulate_fine_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                              np.int32_t colidx,
 *                                              np.uint32_t fineidx):
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":461
 * ############################################################################
 * 
 * cdef inline void accumulate(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int i
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_accumulate(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("accumulate", 0);

  /* "centrosome/_filter.pyx":466
 *         int j
 *         np.int32_t accumulator
 *     accumulate_coarse_histogram(ph, ph.current_column)             # <<<<<<<<<<<<<<
 *     deaccumulate_coarse_histogram(ph, ph.current_column)
 * 
 */
  __pyx_f_10centrosome_7_filter_accumulate_coarse_histogram(__pyx_v_ph, __pyx_v_ph->current_column);

  /* "centrosome/_filter.pyx":467
 *         np.int32_t accumulator
 *     accumulate_coarse_histogram(ph, ph.current_column)
 *     deaccumulate_coarse_histogram(ph, ph.current_column)             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_f_10centrosome_7_filter_deaccumulate_coarse_histogram(__pyx_v_ph, __pyx_v_ph->current_column);

  /* "centrosome/_filter.pyx":461
 * ############################################################################
 * 
 * cdef inline void accumulate(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int i
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":489
 * ############################################################################
 * 
 * cdef inline void update_fine(Histograms *ph, int fineidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int first_update_column = ph.last_update_column[fineidx]+1
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_fine(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, int __pyx_v_fineidx) {
  int __pyx_v_first_update_column;
  int __pyx_v_update_limit;
  int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_t_5numpy_int32_t __pyx_t_3;
  __Pyx_RefNannySetupContext("update_fine", 0);

  /* "centrosome/_filter.pyx":491
 * cdef inline void update_fine(Histograms *ph, int fineidx):
 *     cdef:
 *         int first_update_column = ph.last_update_column[fineidx]+1             # <<<<<<<<<<<<<<
 *         int update_limit        = ph.current_column+1
 *         int i
 */
  __pyx_v_first_update_column = ((__pyx_v_ph->last_update_column[__pyx_v_fineidx]) + 1);

  /* "centrosome/_filter.pyx":492
 *     cdef:
 *         int first_update_column = ph.last_update_column[fineidx]+1
 *         int update_limit        = ph.current_column+1             # <<<<<<<<<<<<<<
 *         int i
 * 
 */
  __pyx_v_update_limit = (__pyx_v_ph->current_column + 1);

  /* "centrosome/_filter.pyx":495
 *         int i
 * 
 *     for i in range(first_update_column, update_limit):             # <<<<<<<<<<<<<<
 *         accumulate_fine_histogram(ph, i, fineidx)
 *         deaccumulate_fine_histogram(ph, i, fineidx)
 */
  __pyx_t_1 = __pyx_v_update_limit;
  for (__pyx_t_2 = __pyx_v_first_update_column; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "centrosome/_filter.pyx":496
 * 
 *     for i in range(first_update_column, update_limit):
 *         accumulate_fine_histogram(ph, i, fineidx)             # <<<<<<<<<<<<<<
 *         deaccumulate_fine_histogram(ph, i, fineidx)
 *     ph.last_update_column[fineidx] = ph.current_column
 */
    __pyx_f_10centrosome_7_filter_accumulate_fine_histogram(__pyx_v_ph, __pyx_v_i, __pyx_v_fineidx);

    /* "centrosome/_filter.pyx":497
 *     for i in range(first_update_column, update_limit):
 *         accumulate_fine_histogram(ph, i, fineidx)
 *         deaccumulate_fine_histogram(ph, i, fineidx)             # <<<<<<<<<<<<<<
 *     ph.last_update_column[fineidx] = ph.current_column
 * 
 */
    __pyx_f_10centrosome_7_filter_deaccumulate_fine_histogram(__pyx_v_ph, __pyx_v_i, __pyx_v_fineidx);
  }

  /* "centrosome/_filter.pyx":498
 *         accumulate_fine_histogram(ph, i, fineidx)
 *         deaccumulate_fine_histogram(ph, i, fineidx)
 *     ph.last_update_column[fineidx] = ph.current_column             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_t_3 = __pyx_v_ph->current_column;
  (__pyx_v_ph->last_update_column[__pyx_v_fineidx]) = __pyx_t_3;

  /* "centrosome/_filter.pyx":489
 * ############################################################################
 * 
 * cdef inline void update_fine(Histograms *ph, int fineidx):             # <<<<<<<<<<<<<<
 *     cdef:
 *         int first_update_column = ph.last_update_column[fineidx]+1
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":512
 * #
 * ############################################################################
 * cdef inline void update_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                   HistogramPiece *hist_piece,
 *                                   pixel_count_t *pixel_count,
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_histogram(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph, struct __pyx_t_10centrosome_7_filter_HistogramPiece *__pyx_v_hist_piece, __pyx_t_10centrosome_7_filter_pixel_count_t *__pyx_v_pixel_count, struct __pyx_t_10centrosome_7_filter_SCoord *__pyx_v_last_coord, struct __pyx_t_10centrosome_7_filter_SCoord *__pyx_v_coord) {
  __pyx_t_5numpy_int32_t __pyx_v_current_column;
  __pyx_t_5numpy_int32_t __pyx_v_current_row;
  __pyx_t_5numpy_int32_t __pyx_v_current_stride;
  __pyx_t_5numpy_int32_t __pyx_v_column_count;
  __pyx_t_5numpy_int32_t __pyx_v_row_count;
  __pyx_t_5numpy_uint8_t __pyx_v_value;
  __pyx_t_5numpy_int32_t __pyx_v_stride;
  __pyx_t_5numpy_int32_t __pyx_v_x;
  __pyx_t_5numpy_int32_t __pyx_v_y;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_int32_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  __pyx_t_5numpy_uint8_t __pyx_t_5;
  __Pyx_RefNannySetupContext("update_histogram", 0);

  /* "centrosome/_filter.pyx":518
 *                                   SCoord *coord):
 *     cdef:
 *         np.int32_t current_column = ph.current_column             # <<<<<<<<<<<<<<
 *         np.int32_t current_row    = ph.current_row
 *         np.int32_t current_stride = ph.current_stride
 */
  __pyx_t_1 = __pyx_v_ph->current_column;
  __pyx_v_current_column = __pyx_t_1;

  /* "centrosome/_filter.pyx":519
 *     cdef:
 *         np.int32_t current_column = ph.current_column
 *         np.int32_t current_row    = ph.current_row             # <<<<<<<<<<<<<<
 *         np.int32_t current_stride = ph.current_stride
 *         np.int32_t column_count   = ph.column_count
 */
  __pyx_t_1 = __pyx_v_ph->current_row;
  __pyx_v_current_row = __pyx_t_1;

  /* "centrosome/_filter.pyx":520
 *         np.int32_t current_column = ph.current_column
 *         np.int32_t current_row    = ph.current_row
 *         np.int32_t current_stride = ph.current_stride             # <<<<<<<<<<<<<<
 *         np.int32_t column_count   = ph.column_count
 *         np.int32_t row_count      = ph.row_count
 */
  __pyx_t_1 = __pyx_v_ph->current_stride;
  __pyx_v_current_stride = __pyx_t_1;

  /* "centrosome/_filter.pyx":521
 *         np.int32_t current_row    = ph.current_row
 *         np.int32_t current_stride = ph.current_stride
 *         np.int32_t column_count   = ph.column_count             # <<<<<<<<<<<<<<
 *         np.int32_t row_count      = ph.row_count
 *         np.uint8_t value
 */
  __pyx_t_1 = __pyx_v_ph->column_count;
  __pyx_v_column_count = __pyx_t_1;

  /* "centrosome/_filter.pyx":522
 *         np.int32_t current_stride = ph.current_stride
 *         np.int32_t column_count   = ph.column_count
 *         np.int32_t row_count      = ph.row_count             # <<<<<<<<<<<<<<
 *         np.uint8_t value
 *         np.int32_t stride
 */
  __pyx_t_1 = __pyx_v_ph->row_count;
  __pyx_v_row_count = __pyx_t_1;

  /* "centrosome/_filter.pyx":528
 *         np.int32_t y
 * 
 *     x      = last_coord.x + current_column             # <<<<<<<<<<<<<<
 *     y      = last_coord.y + current_row
 *     stride = current_stride+last_coord.stride
 */
  __pyx_v_x = (__pyx_v_last_coord->x + __pyx_v_current_column);

  /* "centrosome/_filter.pyx":529
 * 
 *     x      = last_coord.x + current_column
 *     y      = last_coord.y + current_row             # <<<<<<<<<<<<<<
 *     stride = current_stride+last_coord.stride
 * 
 */
  __pyx_v_y = (__pyx_v_last_coord->y + __pyx_v_current_row);

  /* "centrosome/_filter.pyx":530
 *     x      = last_coord.x + current_column
 *     y      = last_coord.y + current_row
 *     stride = current_stride+last_coord.stride             # <<<<<<<<<<<<<<
 * 
 *     if (x >= 0 and x < column_count and
 */
  __pyx_v_stride = (__pyx_v_current_stride + __pyx_v_last_coord->stride);

  /* "centrosome/_filter.pyx":532
 *     stride = current_stride+last_coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  __pyx_t_3 = ((__pyx_v_x >= 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_x < __pyx_v_column_count) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }

  /* "centrosome/_filter.pyx":533
 * 
 *     if (x >= 0 and x < column_count and
 *         y >= 0 and y < row_count and             # <<<<<<<<<<<<<<
 *         ph.mask[stride]):
 *         value = ph.data[stride]
 */
  __pyx_t_3 = ((__pyx_v_y >= 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_y < __pyx_v_row_count) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }

  /* "centrosome/_filter.pyx":534
 *     if (x >= 0 and x < column_count and
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):             # <<<<<<<<<<<<<<
 *         value = ph.data[stride]
 *         pixel_count[0] -= 1
 */
  __pyx_t_3 = ((__pyx_v_ph->mask[__pyx_v_stride]) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;

  /* "centrosome/_filter.pyx":532
 *     stride = current_stride+last_coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  if (__pyx_t_2) {

    /* "centrosome/_filter.pyx":535
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 *         value = ph.data[stride]             # <<<<<<<<<<<<<<
 *         pixel_count[0] -= 1
 *         hist_piece.fine[value] -= 1
 */
    __pyx_v_value = (__pyx_v_ph->data[__pyx_v_stride]);

    /* "centrosome/_filter.pyx":536
 *         ph.mask[stride]):
 *         value = ph.data[stride]
 *         pixel_count[0] -= 1             # <<<<<<<<<<<<<<
 *         hist_piece.fine[value] -= 1
 *         hist_piece.coarse[value / 16] -= 1
 */
    __pyx_t_4 = 0;
    (__pyx_v_pixel_count[__pyx_t_4]) = ((__pyx_v_pixel_count[__pyx_t_4]) - 1);

    /* "centrosome/_filter.pyx":537
 *         value = ph.data[stride]
 *         pixel_count[0] -= 1
 *         hist_piece.fine[value] -= 1             # <<<<<<<<<<<<<<
 *         hist_piece.coarse[value / 16] -= 1
 * 
 */
    __pyx_t_5 = __pyx_v_value;
    (__pyx_v_hist_piece->fine[__pyx_t_5]) = ((__pyx_v_hist_piece->fine[__pyx_t_5]) - 1);

    /* "centrosome/_filter.pyx":538
 *         pixel_count[0] -= 1
 *         hist_piece.fine[value] -= 1
 *         hist_piece.coarse[value / 16] -= 1             # <<<<<<<<<<<<<<
 * 
 *     x      = coord.x + current_column
 */
    __pyx_t_4 = __Pyx_div_long(__pyx_v_value, 16);
    (__pyx_v_hist_piece->coarse[__pyx_t_4]) = ((__pyx_v_hist_piece->coarse[__pyx_t_4]) - 1);

    /* "centrosome/_filter.pyx":532
 *     stride = current_stride+last_coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  }

  /* "centrosome/_filter.pyx":540
 *         hist_piece.coarse[value / 16] -= 1
 * 
 *     x      = coord.x + current_column             # <<<<<<<<<<<<<<
 *     y      = coord.y + current_row
 *     stride = current_stride + coord.stride
 */
  __pyx_v_x = (__pyx_v_coord->x + __pyx_v_current_column);

  /* "centrosome/_filter.pyx":541
 * 
 *     x      = coord.x + current_column
 *     y      = coord.y + current_row             # <<<<<<<<<<<<<<
 *     stride = current_stride + coord.stride
 * 
 */
  __pyx_v_y = (__pyx_v_coord->y + __pyx_v_current_row);

  /* "centrosome/_filter.pyx":542
 *     x      = coord.x + current_column
 *     y      = coord.y + current_row
 *     stride = current_stride + coord.stride             # <<<<<<<<<<<<<<
 * 
 *     if (x >= 0 and x < column_count and
 */
  __pyx_v_stride = (__pyx_v_current_stride + __pyx_v_coord->stride);

  /* "centrosome/_filter.pyx":544
 *     stride = current_stride + coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  __pyx_t_3 = ((__pyx_v_x >= 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_x < __pyx_v_column_count) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L10_bool_binop_done;
  }

  /* "centrosome/_filter.pyx":545
 * 
 *     if (x >= 0 and x < column_count and
 *         y >= 0 and y < row_count and             # <<<<<<<<<<<<<<
 *         ph.mask[stride]):
 *         value = ph.data[stride]
 */
  __pyx_t_3 = ((__pyx_v_y >= 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_y < __pyx_v_row_count) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L10_bool_binop_done;
  }

  /* "centrosome/_filter.pyx":546
 *     if (x >= 0 and x < column_count and
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):             # <<<<<<<<<<<<<<
 *         value = ph.data[stride]
 *         pixel_count[0] += 1
 */
  __pyx_t_3 = ((__pyx_v_ph->mask[__pyx_v_stride]) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L10_bool_binop_done:;

  /* "centrosome/_filter.pyx":544
 *     stride = current_stride + coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  if (__pyx_t_2) {

    /* "centrosome/_filter.pyx":547
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 *         value = ph.data[stride]             # <<<<<<<<<<<<<<
 *         pixel_count[0] += 1
 *         hist_piece.fine[value] += 1
 */
    __pyx_v_value = (__pyx_v_ph->data[__pyx_v_stride]);

    /* "centrosome/_filter.pyx":548
 *         ph.mask[stride]):
 *         value = ph.data[stride]
 *         pixel_count[0] += 1             # <<<<<<<<<<<<<<
 *         hist_piece.fine[value] += 1
 *         hist_piece.coarse[value / 16] += 1
 */
    __pyx_t_4 = 0;
    (__pyx_v_pixel_count[__pyx_t_4]) = ((__pyx_v_pixel_count[__pyx_t_4]) + 1);

    /* "centrosome/_filter.pyx":549
 *         value = ph.data[stride]
 *         pixel_count[0] += 1
 *         hist_piece.fine[value] += 1             # <<<<<<<<<<<<<<
 *         hist_piece.coarse[value / 16] += 1
 * 
 */
    __pyx_t_5 = __pyx_v_value;
    (__pyx_v_hist_piece->fine[__pyx_t_5]) = ((__pyx_v_hist_piece->fine[__pyx_t_5]) + 1);

    /* "centrosome/_filter.pyx":550
 *         pixel_count[0] += 1
 *         hist_piece.fine[value] += 1
 *         hist_piece.coarse[value / 16] += 1             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
    __pyx_t_4 = __Pyx_div_long(__pyx_v_value, 16);
    (__pyx_v_hist_piece->coarse[__pyx_t_4]) = ((__pyx_v_hist_piece->coarse[__pyx_t_4]) + 1);

    /* "centrosome/_filter.pyx":544
 *     stride = current_stride + coord.stride
 * 
 *     if (x >= 0 and x < column_count and             # <<<<<<<<<<<<<<
 *         y >= 0 and y < row_count and
 *         ph.mask[stride]):
 */
  }

  /* "centrosome/_filter.pyx":512
 * #
 * ############################################################################
 * cdef inline void update_histogram(Histograms *ph,             # <<<<<<<<<<<<<<
 *                                   HistogramPiece *hist_piece,
 *                                   pixel_count_t *pixel_count,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":557
 * #
 * ############################################################################
 * cdef inline void update_current_location(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.int32_t current_column   = ph.current_column
 */

static CYTHON_INLINE void __pyx_f_10centrosome_7_filter_update_current_location(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph) {
  __pyx_t_5numpy_int32_t __pyx_v_current_column;
  CYTHON_UNUSED __pyx_t_5numpy_int32_t __pyx_v_radius;
  __pyx_t_5numpy_int32_t __pyx_v_top_left_off;
  __pyx_t_5numpy_int32_t __pyx_v_top_right_off;
  __pyx_t_5numpy_int32_t __pyx_v_bottom_left_off;
  __pyx_t_5numpy_int32_t __pyx_v_bottom_right_off;
  __pyx_t_5numpy_int32_t __pyx_v_leading_edge_off;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_int32_t __pyx_t_1;
  __Pyx_RefNannySetupContext("update_current_location", 0);

  /* "centrosome/_filter.pyx":559
 * cdef inline void update_current_location(Histograms *ph):
 *     cdef:
 *         np.int32_t current_column   = ph.current_column             # <<<<<<<<<<<<<<
 *         np.int32_t radius           = ph.radius
 *         np.int32_t top_left_off     = tl_br_colidx(ph, current_column)
 */
  __pyx_t_1 = __pyx_v_ph->current_column;
  __pyx_v_current_column = __pyx_t_1;

  /* "centrosome/_filter.pyx":560
 *     cdef:
 *         np.int32_t current_column   = ph.current_column
 *         np.int32_t radius           = ph.radius             # <<<<<<<<<<<<<<
 *         np.int32_t top_left_off     = tl_br_colidx(ph, current_column)
 *         np.int32_t top_right_off    = tr_bl_colidx(ph, current_column)
 */
  __pyx_t_1 = __pyx_v_ph->radius;
  __pyx_v_radius = __pyx_t_1;

  /* "centrosome/_filter.pyx":561
 *         np.int32_t current_column   = ph.current_column
 *         np.int32_t radius           = ph.radius
 *         np.int32_t top_left_off     = tl_br_colidx(ph, current_column)             # <<<<<<<<<<<<<<
 *         np.int32_t top_right_off    = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_left_off  = tr_bl_colidx(ph, current_column)
 */
  __pyx_v_top_left_off = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_current_column);

  /* "centrosome/_filter.pyx":562
 *         np.int32_t radius           = ph.radius
 *         np.int32_t top_left_off     = tl_br_colidx(ph, current_column)
 *         np.int32_t top_right_off    = tr_bl_colidx(ph, current_column)             # <<<<<<<<<<<<<<
 *         np.int32_t bottom_left_off  = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_right_off = tl_br_colidx(ph, current_column)
 */
  __pyx_v_top_right_off = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_current_column);

  /* "centrosome/_filter.pyx":563
 *         np.int32_t top_left_off     = tl_br_colidx(ph, current_column)
 *         np.int32_t top_right_off    = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_left_off  = tr_bl_colidx(ph, current_column)             # <<<<<<<<<<<<<<
 *         np.int32_t bottom_right_off = tl_br_colidx(ph, current_column)
 *         np.int32_t leading_edge_off = leading_edge_colidx(ph, current_column)
 */
  __pyx_v_bottom_left_off = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, __pyx_v_current_column);

  /* "centrosome/_filter.pyx":564
 *         np.int32_t top_right_off    = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_left_off  = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_right_off = tl_br_colidx(ph, current_column)             # <<<<<<<<<<<<<<
 *         np.int32_t leading_edge_off = leading_edge_colidx(ph, current_column)
 *         np.int32_t *coarse_histogram
 */
  __pyx_v_bottom_right_off = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, __pyx_v_current_column);

  /* "centrosome/_filter.pyx":565
 *         np.int32_t bottom_left_off  = tr_bl_colidx(ph, current_column)
 *         np.int32_t bottom_right_off = tl_br_colidx(ph, current_column)
 *         np.int32_t leading_edge_off = leading_edge_colidx(ph, current_column)             # <<<<<<<<<<<<<<
 *         np.int32_t *coarse_histogram
 *         np.int32_t *fine_histogram
 */
  __pyx_v_leading_edge_off = __pyx_f_10centrosome_7_filter_leading_edge_colidx(__pyx_v_ph, __pyx_v_current_column);

  /* "centrosome/_filter.pyx":575
 *         np.int32_t stride
 * 
 *     update_histogram(ph, &ph.histogram[top_left_off].top_left,             # <<<<<<<<<<<<<<
 *                      &ph.pixel_count[top_left_off].top_left,
 *                      &ph.last_top_left,
 */
  __pyx_f_10centrosome_7_filter_update_histogram(__pyx_v_ph, (&(__pyx_v_ph->histogram[__pyx_v_top_left_off]).top_left), (&(__pyx_v_ph->pixel_count[__pyx_v_top_left_off]).top_left), (&__pyx_v_ph->last_top_left), (&__pyx_v_ph->top_left));

  /* "centrosome/_filter.pyx":580
 *                      &ph.top_left)
 * 
 *     update_histogram(ph, &ph.histogram[top_right_off].top_right,             # <<<<<<<<<<<<<<
 *                      &ph.pixel_count[top_right_off].top_right,
 *                      &ph.last_top_right,
 */
  __pyx_f_10centrosome_7_filter_update_histogram(__pyx_v_ph, (&(__pyx_v_ph->histogram[__pyx_v_top_right_off]).top_right), (&(__pyx_v_ph->pixel_count[__pyx_v_top_right_off]).top_right), (&__pyx_v_ph->last_top_right), (&__pyx_v_ph->top_right));

  /* "centrosome/_filter.pyx":585
 *                      &ph.top_right)
 * 
 *     update_histogram(ph, &ph.histogram[bottom_left_off].bottom_left,             # <<<<<<<<<<<<<<
 *                      &ph.pixel_count[bottom_left_off].bottom_left,
 *                      &ph.last_bottom_left,
 */
  __pyx_f_10centrosome_7_filter_update_histogram(__pyx_v_ph, (&(__pyx_v_ph->histogram[__pyx_v_bottom_left_off]).bottom_left), (&(__pyx_v_ph->pixel_count[__pyx_v_bottom_left_off]).bottom_left), (&__pyx_v_ph->last_bottom_left), (&__pyx_v_ph->bottom_left));

  /* "centrosome/_filter.pyx":590
 *                      &ph.bottom_left)
 * 
 *     update_histogram(ph, &ph.histogram[bottom_right_off].bottom_right,             # <<<<<<<<<<<<<<
 *                      &ph.pixel_count[bottom_right_off].bottom_right,
 *                      &ph.last_bottom_right,
 */
  __pyx_f_10centrosome_7_filter_update_histogram(__pyx_v_ph, (&(__pyx_v_ph->histogram[__pyx_v_bottom_right_off]).bottom_right), (&(__pyx_v_ph->pixel_count[__pyx_v_bottom_right_off]).bottom_right), (&__pyx_v_ph->last_bottom_right), (&__pyx_v_ph->bottom_right));

  /* "centrosome/_filter.pyx":595
 *                      &ph.bottom_right)
 * 
 *     update_histogram(ph, &ph.histogram[leading_edge_off].edge,             # <<<<<<<<<<<<<<
 *                      &ph.pixel_count[leading_edge_off].edge,
 *                      &ph.last_leading_edge,
 */
  __pyx_f_10centrosome_7_filter_update_histogram(__pyx_v_ph, (&(__pyx_v_ph->histogram[__pyx_v_leading_edge_off]).edge), (&(__pyx_v_ph->pixel_count[__pyx_v_leading_edge_off]).edge), (&__pyx_v_ph->last_leading_edge), (&__pyx_v_ph->leading_edge));

  /* "centrosome/_filter.pyx":557
 * #
 * ############################################################################
 * cdef inline void update_current_location(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.int32_t current_column   = ph.current_column
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "centrosome/_filter.pyx":606
 * ############################################################################
 * 
 * cdef inline np.uint8_t find_median(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.uint32_t pixels_below      # of pixels below the median
 */

static CYTHON_INLINE __pyx_t_5numpy_uint8_t __pyx_f_10centrosome_7_filter_find_median(struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph) {
  __pyx_t_5numpy_uint32_t __pyx_v_pixels_below;
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_5numpy_uint32_t __pyx_v_accumulator;
  __pyx_t_5numpy_uint8_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  long __pyx_t_3;
  __Pyx_RefNannySetupContext("find_median", 0);

  /* "centrosome/_filter.pyx":614
 *         np.uint32_t accumulator
 * 
 *     if ph.accumulator_count == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 */
  __pyx_t_1 = ((__pyx_v_ph->accumulator_count == 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":615
 * 
 *     if ph.accumulator_count == 0:
 *         return 0             # <<<<<<<<<<<<<<
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 *     if pixels_below > 0:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "centrosome/_filter.pyx":614
 *         np.uint32_t accumulator
 * 
 *     if ph.accumulator_count == 0:             # <<<<<<<<<<<<<<
 *         return 0
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 */
  }

  /* "centrosome/_filter.pyx":616
 *     if ph.accumulator_count == 0:
 *         return 0
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff             # <<<<<<<<<<<<<<
 *     if pixels_below > 0:
 *         pixels_below -= 1
 */
  __pyx_v_pixels_below = __Pyx_div_long(((__pyx_v_ph->accumulator_count * __pyx_v_ph->percent) + 50), 0x64);

  /* "centrosome/_filter.pyx":617
 *         return 0
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 *     if pixels_below > 0:             # <<<<<<<<<<<<<<
 *         pixels_below -= 1
 *     accumulator = 0
 */
  __pyx_t_1 = ((__pyx_v_pixels_below > 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":618
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 *     if pixels_below > 0:
 *         pixels_below -= 1             # <<<<<<<<<<<<<<
 *     accumulator = 0
 *     for i in range(16):
 */
    __pyx_v_pixels_below = (__pyx_v_pixels_below - 1);

    /* "centrosome/_filter.pyx":617
 *         return 0
 *     pixels_below = (ph.accumulator_count * ph.percent + 50) / 100 # +50 for roundoff
 *     if pixels_below > 0:             # <<<<<<<<<<<<<<
 *         pixels_below -= 1
 *     accumulator = 0
 */
  }

  /* "centrosome/_filter.pyx":619
 *     if pixels_below > 0:
 *         pixels_below -= 1
 *     accumulator = 0             # <<<<<<<<<<<<<<
 *     for i in range(16):
 *         accumulator += ph.accumulator.coarse[i]
 */
  __pyx_v_accumulator = 0;

  /* "centrosome/_filter.pyx":620
 *         pixels_below -= 1
 *     accumulator = 0
 *     for i in range(16):             # <<<<<<<<<<<<<<
 *         accumulator += ph.accumulator.coarse[i]
 *         if accumulator > pixels_below:
 */
  for (__pyx_t_2 = 0; __pyx_t_2 < 16; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "centrosome/_filter.pyx":621
 *     accumulator = 0
 *     for i in range(16):
 *         accumulator += ph.accumulator.coarse[i]             # <<<<<<<<<<<<<<
 *         if accumulator > pixels_below:
 *             break
 */
    __pyx_v_accumulator = (__pyx_v_accumulator + (__pyx_v_ph->accumulator.coarse[__pyx_v_i]));

    /* "centrosome/_filter.pyx":622
 *     for i in range(16):
 *         accumulator += ph.accumulator.coarse[i]
 *         if accumulator > pixels_below:             # <<<<<<<<<<<<<<
 *             break
 *     accumulator -= ph.accumulator.coarse[i]
 */
    __pyx_t_1 = ((__pyx_v_accumulator > __pyx_v_pixels_below) != 0);
    if (__pyx_t_1) {

      /* "centrosome/_filter.pyx":623
 *         accumulator += ph.accumulator.coarse[i]
 *         if accumulator > pixels_below:
 *             break             # <<<<<<<<<<<<<<
 *     accumulator -= ph.accumulator.coarse[i]
 *     update_fine(ph, i)
 */
      goto __pyx_L6_break;

      /* "centrosome/_filter.pyx":622
 *     for i in range(16):
 *         accumulator += ph.accumulator.coarse[i]
 *         if accumulator > pixels_below:             # <<<<<<<<<<<<<<
 *             break
 *     accumulator -= ph.accumulator.coarse[i]
 */
    }
  }
  __pyx_L6_break:;

  /* "centrosome/_filter.pyx":624
 *         if accumulator > pixels_below:
 *             break
 *     accumulator -= ph.accumulator.coarse[i]             # <<<<<<<<<<<<<<
 *     update_fine(ph, i)
 *     for j in range(i*16,(i+1)*16):
 */
  __pyx_v_accumulator = (__pyx_v_accumulator - (__pyx_v_ph->accumulator.coarse[__pyx_v_i]));

  /* "centrosome/_filter.pyx":625
 *             break
 *     accumulator -= ph.accumulator.coarse[i]
 *     update_fine(ph, i)             # <<<<<<<<<<<<<<
 *     for j in range(i*16,(i+1)*16):
 *         accumulator += ph.accumulator.fine[j]
 */
  __pyx_f_10centrosome_7_filter_update_fine(__pyx_v_ph, __pyx_v_i);

  /* "centrosome/_filter.pyx":626
 *     accumulator -= ph.accumulator.coarse[i]
 *     update_fine(ph, i)
 *     for j in range(i*16,(i+1)*16):             # <<<<<<<<<<<<<<
 *         accumulator += ph.accumulator.fine[j]
 *         if accumulator > pixels_below:
 */
  __pyx_t_3 = ((__pyx_v_i + 1) * 16);
  for (__pyx_t_2 = (__pyx_v_i * 16); __pyx_t_2 < __pyx_t_3; __pyx_t_2+=1) {
    __pyx_v_j = __pyx_t_2;

    /* "centrosome/_filter.pyx":627
 *     update_fine(ph, i)
 *     for j in range(i*16,(i+1)*16):
 *         accumulator += ph.accumulator.fine[j]             # <<<<<<<<<<<<<<
 *         if accumulator > pixels_below:
 *             return <np.uint8_t> j
 */
    __pyx_v_accumulator = (__pyx_v_accumulator + (__pyx_v_ph->accumulator.fine[__pyx_v_j]));

    /* "centrosome/_filter.pyx":628
 *     for j in range(i*16,(i+1)*16):
 *         accumulator += ph.accumulator.fine[j]
 *         if accumulator > pixels_below:             # <<<<<<<<<<<<<<
 *             return <np.uint8_t> j
 *     return 0
 */
    __pyx_t_1 = ((__pyx_v_accumulator > __pyx_v_pixels_below) != 0);
    if (__pyx_t_1) {

      /* "centrosome/_filter.pyx":629
 *         accumulator += ph.accumulator.fine[j]
 *         if accumulator > pixels_below:
 *             return <np.uint8_t> j             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
      __pyx_r = ((__pyx_t_5numpy_uint8_t)__pyx_v_j);
      goto __pyx_L0;

      /* "centrosome/_filter.pyx":628
 *     for j in range(i*16,(i+1)*16):
 *         accumulator += ph.accumulator.fine[j]
 *         if accumulator > pixels_below:             # <<<<<<<<<<<<<<
 *             return <np.uint8_t> j
 *     return 0
 */
    }
  }

  /* "centrosome/_filter.pyx":630
 *         if accumulator > pixels_below:
 *             return <np.uint8_t> j
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * ############################################################################
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":606
 * ############################################################################
 * 
 * cdef inline np.uint8_t find_median(Histograms *ph):             # <<<<<<<<<<<<<<
 *     cdef:
 *         np.uint32_t pixels_below      # of pixels below the median
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":647
 * #
 * ############################################################################
 * cdef int c_median_filter(np.int32_t   rows,             # <<<<<<<<<<<<<<
 *                          np.int32_t   columns,
 *                          np.int32_t   row_stride,
 */

static int __pyx_f_10centrosome_7_filter_c_median_filter(__pyx_t_5numpy_int32_t __pyx_v_rows, __pyx_t_5numpy_int32_t __pyx_v_columns, __pyx_t_5numpy_int32_t __pyx_v_row_stride, __pyx_t_5numpy_int32_t __pyx_v_col_stride, __pyx_t_5numpy_int32_t __pyx_v_radius, __pyx_t_5numpy_int32_t __pyx_v_percent, __pyx_t_5numpy_uint8_t *__pyx_v_data, __pyx_t_5numpy_uint8_t *__pyx_v_mask, __pyx_t_5numpy_uint8_t *__pyx_v_output) {
  struct __pyx_t_10centrosome_7_filter_Histograms *__pyx_v_ph;
  int __pyx_v_row;
  int __pyx_v_col;
  int __pyx_v_i;
  __pyx_t_5numpy_int32_t __pyx_v_tl_br_off;
  __pyx_t_5numpy_int32_t __pyx_v_tr_bl_off;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __pyx_t_5numpy_int32_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  long __pyx_t_5;
  __pyx_t_5numpy_int32_t __pyx_t_6;
  __Pyx_RefNannySetupContext("c_median_filter", 0);

  /* "centrosome/_filter.pyx":667
 *         np.int32_t bottom_right_off
 * 
 *     ph = allocate_histograms(rows, columns, row_stride, col_stride,             # <<<<<<<<<<<<<<
 *                              radius, percent, data, mask, output)
 *     if not ph:
 */
  __pyx_v_ph = __pyx_f_10centrosome_7_filter_allocate_histograms(__pyx_v_rows, __pyx_v_columns, __pyx_v_row_stride, __pyx_v_col_stride, __pyx_v_radius, __pyx_v_percent, __pyx_v_data, __pyx_v_mask, __pyx_v_output);

  /* "centrosome/_filter.pyx":669
 *     ph = allocate_histograms(rows, columns, row_stride, col_stride,
 *                              radius, percent, data, mask, output)
 *     if not ph:             # <<<<<<<<<<<<<<
 *        return 1
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_ph != 0)) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":670
 *                              radius, percent, data, mask, output)
 *     if not ph:
 *        return 1             # <<<<<<<<<<<<<<
 * 
 *     for row in range(-radius, rows):
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "centrosome/_filter.pyx":669
 *     ph = allocate_histograms(rows, columns, row_stride, col_stride,
 *                              radius, percent, data, mask, output)
 *     if not ph:             # <<<<<<<<<<<<<<
 *        return 1
 * 
 */
  }

  /* "centrosome/_filter.pyx":672
 *        return 1
 * 
 *     for row in range(-radius, rows):             # <<<<<<<<<<<<<<
 *         #
 *         # Initialize the starting diagonal histograms to zero. The leading
 */
  __pyx_t_2 = __pyx_v_rows;
  for (__pyx_t_3 = (-__pyx_v_radius); __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_row = __pyx_t_3;

    /* "centrosome/_filter.pyx":681
 *         # start of each row
 *         #
 *         tl_br_off     = tl_br_colidx(ph, -radius)             # <<<<<<<<<<<<<<
 *         tr_bl_off     = tr_bl_colidx(ph, columns+radius-1)
 * 
 */
    __pyx_v_tl_br_off = __pyx_f_10centrosome_7_filter_tl_br_colidx(__pyx_v_ph, (-__pyx_v_radius));

    /* "centrosome/_filter.pyx":682
 *         #
 *         tl_br_off     = tl_br_colidx(ph, -radius)
 *         tr_bl_off     = tr_bl_colidx(ph, columns+radius-1)             # <<<<<<<<<<<<<<
 * 
 *         memset(&ph.histogram[tl_br_off].top_left, 0, sizeof(HistogramPiece))
 */
    __pyx_v_tr_bl_off = __pyx_f_10centrosome_7_filter_tr_bl_colidx(__pyx_v_ph, ((__pyx_v_columns + __pyx_v_radius) - 1));

    /* "centrosome/_filter.pyx":684
 *         tr_bl_off     = tr_bl_colidx(ph, columns+radius-1)
 * 
 *         memset(&ph.histogram[tl_br_off].top_left, 0, sizeof(HistogramPiece))             # <<<<<<<<<<<<<<
 *         memset(&ph.histogram[tl_br_off].bottom_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].top_right, 0, sizeof(HistogramPiece))
 */
    memset((&(__pyx_v_ph->histogram[__pyx_v_tl_br_off]).top_left), 0, (sizeof(struct __pyx_t_10centrosome_7_filter_HistogramPiece)));

    /* "centrosome/_filter.pyx":685
 * 
 *         memset(&ph.histogram[tl_br_off].top_left, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tl_br_off].bottom_right, 0, sizeof(HistogramPiece))             # <<<<<<<<<<<<<<
 *         memset(&ph.histogram[tr_bl_off].top_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].bottom_left, 0, sizeof(HistogramPiece))
 */
    memset((&(__pyx_v_ph->histogram[__pyx_v_tl_br_off]).bottom_right), 0, (sizeof(struct __pyx_t_10centrosome_7_filter_HistogramPiece)));

    /* "centrosome/_filter.pyx":686
 *         memset(&ph.histogram[tl_br_off].top_left, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tl_br_off].bottom_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].top_right, 0, sizeof(HistogramPiece))             # <<<<<<<<<<<<<<
 *         memset(&ph.histogram[tr_bl_off].bottom_left, 0, sizeof(HistogramPiece))
 *         ph.pixel_count[tl_br_off].top_left     = 0
 */
    memset((&(__pyx_v_ph->histogram[__pyx_v_tr_bl_off]).top_right), 0, (sizeof(struct __pyx_t_10centrosome_7_filter_HistogramPiece)));

    /* "centrosome/_filter.pyx":687
 *         memset(&ph.histogram[tl_br_off].bottom_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].top_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].bottom_left, 0, sizeof(HistogramPiece))             # <<<<<<<<<<<<<<
 *         ph.pixel_count[tl_br_off].top_left     = 0
 *         ph.pixel_count[tl_br_off].bottom_right = 0
 */
    memset((&(__pyx_v_ph->histogram[__pyx_v_tr_bl_off]).bottom_left), 0, (sizeof(struct __pyx_t_10centrosome_7_filter_HistogramPiece)));

    /* "centrosome/_filter.pyx":688
 *         memset(&ph.histogram[tr_bl_off].top_right, 0, sizeof(HistogramPiece))
 *         memset(&ph.histogram[tr_bl_off].bottom_left, 0, sizeof(HistogramPiece))
 *         ph.pixel_count[tl_br_off].top_left     = 0             # <<<<<<<<<<<<<<
 *         ph.pixel_count[tl_br_off].bottom_right = 0
 *         ph.pixel_count[tr_bl_off].top_right    = 0
 */
    (__pyx_v_ph->pixel_count[__pyx_v_tl_br_off]).top_left = 0;

    /* "centrosome/_filter.pyx":689
 *         memset(&ph.histogram[tr_bl_off].bottom_left, 0, sizeof(HistogramPiece))
 *         ph.pixel_count[tl_br_off].top_left     = 0
 *         ph.pixel_count[tl_br_off].bottom_right = 0             # <<<<<<<<<<<<<<
 *         ph.pixel_count[tr_bl_off].top_right    = 0
 *         ph.pixel_count[tr_bl_off].bottom_left  = 0
 */
    (__pyx_v_ph->pixel_count[__pyx_v_tl_br_off]).bottom_right = 0;

    /* "centrosome/_filter.pyx":690
 *         ph.pixel_count[tl_br_off].top_left     = 0
 *         ph.pixel_count[tl_br_off].bottom_right = 0
 *         ph.pixel_count[tr_bl_off].top_right    = 0             # <<<<<<<<<<<<<<
 *         ph.pixel_count[tr_bl_off].bottom_left  = 0
 *         #
 */
    (__pyx_v_ph->pixel_count[__pyx_v_tr_bl_off]).top_right = 0;

    /* "centrosome/_filter.pyx":691
 *         ph.pixel_count[tl_br_off].bottom_right = 0
 *         ph.pixel_count[tr_bl_off].top_right    = 0
 *         ph.pixel_count[tr_bl_off].bottom_left  = 0             # <<<<<<<<<<<<<<
 *         #
 *         # Initialize the accumulator (octagon histogram) to zero
 */
    (__pyx_v_ph->pixel_count[__pyx_v_tr_bl_off]).bottom_left = 0;

    /* "centrosome/_filter.pyx":695
 *         # Initialize the accumulator (octagon histogram) to zero
 *         #
 *         memset(&ph.accumulator, 0, sizeof(ph.accumulator))             # <<<<<<<<<<<<<<
 *         ph.accumulator_count = 0
 *         for i in range(16):
 */
    memset((&__pyx_v_ph->accumulator), 0, (sizeof(__pyx_v_ph->accumulator)));

    /* "centrosome/_filter.pyx":696
 *         #
 *         memset(&ph.accumulator, 0, sizeof(ph.accumulator))
 *         ph.accumulator_count = 0             # <<<<<<<<<<<<<<
 *         for i in range(16):
 *             ph.last_update_column[i] = -radius-1
 */
    __pyx_v_ph->accumulator_count = 0;

    /* "centrosome/_filter.pyx":697
 *         memset(&ph.accumulator, 0, sizeof(ph.accumulator))
 *         ph.accumulator_count = 0
 *         for i in range(16):             # <<<<<<<<<<<<<<
 *             ph.last_update_column[i] = -radius-1
 *         #
 */
    for (__pyx_t_4 = 0; __pyx_t_4 < 16; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "centrosome/_filter.pyx":698
 *         ph.accumulator_count = 0
 *         for i in range(16):
 *             ph.last_update_column[i] = -radius-1             # <<<<<<<<<<<<<<
 *         #
 *         # Initialize the current stride to the beginning of the row
 */
      (__pyx_v_ph->last_update_column[__pyx_v_i]) = ((-__pyx_v_radius) - 1);
    }

    /* "centrosome/_filter.pyx":702
 *         # Initialize the current stride to the beginning of the row
 *         #
 *         ph.current_row = row             # <<<<<<<<<<<<<<
 *         #
 *         # Update locations and coarse accumulator for the octagon
 */
    __pyx_v_ph->current_row = __pyx_v_row;

    /* "centrosome/_filter.pyx":707
 *         # for points before 0
 *         #
 *         for col in range(-radius, 0 if row >=0 else columns+radius):             # <<<<<<<<<<<<<<
 *             ph.current_column = col
 *             ph.current_stride = row * row_stride + col * col_stride
 */
    if (((__pyx_v_row >= 0) != 0)) {
      __pyx_t_5 = 0;
    } else {
      __pyx_t_5 = (__pyx_v_columns + __pyx_v_radius);
    }
    for (__pyx_t_4 = (-__pyx_v_radius); __pyx_t_4 < __pyx_t_5; __pyx_t_4+=1) {
      __pyx_v_col = __pyx_t_4;

      /* "centrosome/_filter.pyx":708
 *         #
 *         for col in range(-radius, 0 if row >=0 else columns+radius):
 *             ph.current_column = col             # <<<<<<<<<<<<<<
 *             ph.current_stride = row * row_stride + col * col_stride
 *             update_current_location(ph)
 */
      __pyx_v_ph->current_column = __pyx_v_col;

      /* "centrosome/_filter.pyx":709
 *         for col in range(-radius, 0 if row >=0 else columns+radius):
 *             ph.current_column = col
 *             ph.current_stride = row * row_stride + col * col_stride             # <<<<<<<<<<<<<<
 *             update_current_location(ph)
 *             accumulate(ph)
 */
      __pyx_v_ph->current_stride = ((__pyx_v_row * __pyx_v_row_stride) + (__pyx_v_col * __pyx_v_col_stride));

      /* "centrosome/_filter.pyx":710
 *             ph.current_column = col
 *             ph.current_stride = row * row_stride + col * col_stride
 *             update_current_location(ph)             # <<<<<<<<<<<<<<
 *             accumulate(ph)
 *         #
 */
      __pyx_f_10centrosome_7_filter_update_current_location(__pyx_v_ph);

      /* "centrosome/_filter.pyx":711
 *             ph.current_stride = row * row_stride + col * col_stride
 *             update_current_location(ph)
 *             accumulate(ph)             # <<<<<<<<<<<<<<
 *         #
 *         # Update locations and coarse accumulator and compute
 */
      __pyx_f_10centrosome_7_filter_accumulate(__pyx_v_ph);
    }

    /* "centrosome/_filter.pyx":716
 *         # the median for points between 0 and "columns"
 *         #
 *         if row >= 0:             # <<<<<<<<<<<<<<
 *             for col in range(0, columns):
 *                 ph.current_column = col
 */
    __pyx_t_1 = ((__pyx_v_row >= 0) != 0);
    if (__pyx_t_1) {

      /* "centrosome/_filter.pyx":717
 *         #
 *         if row >= 0:
 *             for col in range(0, columns):             # <<<<<<<<<<<<<<
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride
 */
      __pyx_t_6 = __pyx_v_columns;
      for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_6; __pyx_t_4+=1) {
        __pyx_v_col = __pyx_t_4;

        /* "centrosome/_filter.pyx":718
 *         if row >= 0:
 *             for col in range(0, columns):
 *                 ph.current_column = col             # <<<<<<<<<<<<<<
 *                 ph.current_stride = row * row_stride + col * col_stride
 *                 update_current_location(ph)
 */
        __pyx_v_ph->current_column = __pyx_v_col;

        /* "centrosome/_filter.pyx":719
 *             for col in range(0, columns):
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride             # <<<<<<<<<<<<<<
 *                 update_current_location(ph)
 *                 accumulate(ph)
 */
        __pyx_v_ph->current_stride = ((__pyx_v_row * __pyx_v_row_stride) + (__pyx_v_col * __pyx_v_col_stride));

        /* "centrosome/_filter.pyx":720
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride
 *                 update_current_location(ph)             # <<<<<<<<<<<<<<
 *                 accumulate(ph)
 *                 ph.output[ph.current_stride] = find_median(ph)
 */
        __pyx_f_10centrosome_7_filter_update_current_location(__pyx_v_ph);

        /* "centrosome/_filter.pyx":721
 *                 ph.current_stride = row * row_stride + col * col_stride
 *                 update_current_location(ph)
 *                 accumulate(ph)             # <<<<<<<<<<<<<<
 *                 ph.output[ph.current_stride] = find_median(ph)
 *             for col in range(columns, columns+radius):
 */
        __pyx_f_10centrosome_7_filter_accumulate(__pyx_v_ph);

        /* "centrosome/_filter.pyx":722
 *                 update_current_location(ph)
 *                 accumulate(ph)
 *                 ph.output[ph.current_stride] = find_median(ph)             # <<<<<<<<<<<<<<
 *             for col in range(columns, columns+radius):
 *                 ph.current_column = col
 */
        (__pyx_v_ph->output[__pyx_v_ph->current_stride]) = __pyx_f_10centrosome_7_filter_find_median(__pyx_v_ph);
      }

      /* "centrosome/_filter.pyx":723
 *                 accumulate(ph)
 *                 ph.output[ph.current_stride] = find_median(ph)
 *             for col in range(columns, columns+radius):             # <<<<<<<<<<<<<<
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride
 */
      __pyx_t_6 = (__pyx_v_columns + __pyx_v_radius);
      for (__pyx_t_4 = __pyx_v_columns; __pyx_t_4 < __pyx_t_6; __pyx_t_4+=1) {
        __pyx_v_col = __pyx_t_4;

        /* "centrosome/_filter.pyx":724
 *                 ph.output[ph.current_stride] = find_median(ph)
 *             for col in range(columns, columns+radius):
 *                 ph.current_column = col             # <<<<<<<<<<<<<<
 *                 ph.current_stride = row * row_stride + col * col_stride
 *                 update_current_location(ph)
 */
        __pyx_v_ph->current_column = __pyx_v_col;

        /* "centrosome/_filter.pyx":725
 *             for col in range(columns, columns+radius):
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride             # <<<<<<<<<<<<<<
 *                 update_current_location(ph)
 * 
 */
        __pyx_v_ph->current_stride = ((__pyx_v_row * __pyx_v_row_stride) + (__pyx_v_col * __pyx_v_col_stride));

        /* "centrosome/_filter.pyx":726
 *                 ph.current_column = col
 *                 ph.current_stride = row * row_stride + col * col_stride
 *                 update_current_location(ph)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_f_10centrosome_7_filter_update_current_location(__pyx_v_ph);
      }

      /* "centrosome/_filter.pyx":716
 *         # the median for points between 0 and "columns"
 *         #
 *         if row >= 0:             # <<<<<<<<<<<<<<
 *             for col in range(0, columns):
 *                 ph.current_column = col
 */
    }
  }

  /* "centrosome/_filter.pyx":729
 * 
 * 
 *     free_histograms(ph)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_f_10centrosome_7_filter_free_histograms(__pyx_v_ph);

  /* "centrosome/_filter.pyx":730
 * 
 *     free_histograms(ph)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * def median_filter(np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] data,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":647
 * #
 * ############################################################################
 * cdef int c_median_filter(np.int32_t   rows,             # <<<<<<<<<<<<<<
 *                          np.int32_t   columns,
 *                          np.int32_t   row_stride,
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":732
 *     return 0
 * 
 * def median_filter(np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] output,
 */

/* Python wrapper */
static PyObject *__pyx_pw_10centrosome_7_filter_1median_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10centrosome_7_filter_median_filter[] = "Median filter with octagon shape and masking\n\n    data - a 2d array containing the image data\n    mask - a 2d array of 1=significant pixel, 0=masked\n           similarly shaped to \"data\"\n    output - a 2d array that will hold the output of this operation\n             similarly shaped to \"data\"\n    radius - the radius of the inscribed circle to the octagon\n    percent - sort the unmasked pixels within the octagon into\n              an array (conceptually) and take the value indexed\n              by the size of that array times the percent divided by 100.\n              50 gives the median\n    ";
static PyMethodDef __pyx_mdef_10centrosome_7_filter_1median_filter = {"median_filter", (PyCFunction)__pyx_pw_10centrosome_7_filter_1median_filter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10centrosome_7_filter_median_filter};
static PyObject *__pyx_pw_10centrosome_7_filter_1median_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_data = 0;
  PyArrayObject *__pyx_v_mask = 0;
  PyArrayObject *__pyx_v_output = 0;
  int __pyx_v_radius;
  __pyx_t_5numpy_int32_t __pyx_v_percent;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("median_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_mask,&__pyx_n_s_output,&__pyx_n_s_radius,&__pyx_n_s_percent,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("median_filter", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_output)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("median_filter", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("median_filter", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_percent)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("median_filter", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "median_filter") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_data = ((PyArrayObject *)values[0]);
    __pyx_v_mask = ((PyArrayObject *)values[1]);
    __pyx_v_output = ((PyArrayObject *)values[2]);
    __pyx_v_radius = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_radius == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 735; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_percent = __Pyx_PyInt_As_npy_int32(values[4]); if (unlikely((__pyx_v_percent == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 736; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("median_filter", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("centrosome._filter.median_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), __pyx_ptype_5numpy_ndarray, 1, "data", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_mask), __pyx_ptype_5numpy_ndarray, 1, "mask", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_output), __pyx_ptype_5numpy_ndarray, 1, "output", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 734; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_10centrosome_7_filter_median_filter(__pyx_self, __pyx_v_data, __pyx_v_mask, __pyx_v_output, __pyx_v_radius, __pyx_v_percent);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10centrosome_7_filter_median_filter(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_data, PyArrayObject *__pyx_v_mask, PyArrayObject *__pyx_v_output, int __pyx_v_radius, __pyx_t_5numpy_int32_t __pyx_v_percent) {
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_mask;
  __Pyx_Buffer __pyx_pybuffer_mask;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_output;
  __Pyx_Buffer __pyx_pybuffer_output;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("median_filter", 0);
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;
  __pyx_pybuffer_mask.pybuffer.buf = NULL;
  __pyx_pybuffer_mask.refcount = 0;
  __pyx_pybuffernd_mask.data = NULL;
  __pyx_pybuffernd_mask.rcbuffer = &__pyx_pybuffer_mask;
  __pyx_pybuffer_output.pybuffer.buf = NULL;
  __pyx_pybuffer_output.refcount = 0;
  __pyx_pybuffernd_output.data = NULL;
  __pyx_pybuffernd_output.rcbuffer = &__pyx_pybuffer_output;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_v_data, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_data.diminfo[1].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_data.diminfo[1].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_mask.rcbuffer->pybuffer, (PyObject*)__pyx_v_mask, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_mask.diminfo[0].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_mask.diminfo[0].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_mask.diminfo[1].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_mask.diminfo[1].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_output.rcbuffer->pybuffer, (PyObject*)__pyx_v_output, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_output.diminfo[0].strides = __pyx_pybuffernd_output.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_output.diminfo[0].shape = __pyx_pybuffernd_output.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_output.diminfo[1].strides = __pyx_pybuffernd_output.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_output.diminfo[1].shape = __pyx_pybuffernd_output.rcbuffer->pybuffer.shape[1];

  /* "centrosome/_filter.pyx":750
 *               50 gives the median
 *     """
 *     if percent < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)
 *     if percent > 100:
 */
  __pyx_t_1 = ((__pyx_v_percent < 0) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":751
 *     """
 *     if percent < 0:
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)             # <<<<<<<<<<<<<<
 *     if percent > 100:
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 */
    __pyx_t_2 = __Pyx_PyInt_From_npy_int32(__pyx_v_percent); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Median_filter_percent_d_is_less, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "centrosome/_filter.pyx":750
 *               50 gives the median
 *     """
 *     if percent < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)
 *     if percent > 100:
 */
  }

  /* "centrosome/_filter.pyx":752
 *     if percent < 0:
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)
 *     if percent > 100:             # <<<<<<<<<<<<<<
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 */
  __pyx_t_1 = ((__pyx_v_percent > 0x64) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":753
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)
 *     if percent > 100:
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)             # <<<<<<<<<<<<<<
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 */
    __pyx_t_3 = __Pyx_PyInt_From_npy_int32(__pyx_v_percent); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Median_filter_percent_d_is_great, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 753; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "centrosome/_filter.pyx":752
 *     if percent < 0:
 *         raise ValueError('Median filter percent = %d is less than zero'%percent)
 *     if percent > 100:             # <<<<<<<<<<<<<<
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 */
  }

  /* "centrosome/_filter.pyx":754
 *     if percent > 100:
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 */
  __pyx_t_4 = (((__pyx_v_data->dimensions[0]) != (__pyx_v_mask->dimensions[0])) != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_4 = (((__pyx_v_data->dimensions[1]) != (__pyx_v_mask->dimensions[1])) != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":756
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],             # <<<<<<<<<<<<<<
 *                           mask.shape[0], mask.shape[1]))
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 */
    __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[0])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[1])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "centrosome/_filter.pyx":757
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 *                           mask.shape[0], mask.shape[1]))             # <<<<<<<<<<<<<<
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 */
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_mask->dimensions[0])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_mask->dimensions[1])); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 757; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);

    /* "centrosome/_filter.pyx":756
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],             # <<<<<<<<<<<<<<
 *                           mask.shape[0], mask.shape[1]))
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 */
    __pyx_t_7 = PyTuple_New(4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_t_6);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;

    /* "centrosome/_filter.pyx":755
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%             # <<<<<<<<<<<<<<
 *                          (data.shape[0],data.shape[1],
 *                           mask.shape[0], mask.shape[1]))
 */
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Data_shape_d_d_is_not_mask_shape, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 755; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 755; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 755; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 755; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "centrosome/_filter.pyx":754
 *     if percent > 100:
 *         raise ValueError('Median filter percent = %d is greater than 100'%percent)
 *     if data.shape[0] != mask.shape[0] or data.shape[1] != mask.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Data shape (%d,%d) is not mask shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 */
  }

  /* "centrosome/_filter.pyx":758
 *                          (data.shape[0],data.shape[1],
 *                           mask.shape[0], mask.shape[1]))
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 */
  __pyx_t_4 = (((__pyx_v_data->dimensions[0]) != (__pyx_v_output->dimensions[0])) != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_4 = (((__pyx_v_data->dimensions[1]) != (__pyx_v_output->dimensions[1])) != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":760
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],             # <<<<<<<<<<<<<<
 *                           output.shape[0], output.shape[1]))
 *     if c_median_filter(data.shape[0],   data.shape[1],
 */
    __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[0])); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[1])); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);

    /* "centrosome/_filter.pyx":761
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 *                           output.shape[0], output.shape[1]))             # <<<<<<<<<<<<<<
 *     if c_median_filter(data.shape[0],   data.shape[1],
 *                        data.strides[0], data.strides[1],
 */
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_output->dimensions[0])); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 761; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_output->dimensions[1])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 761; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "centrosome/_filter.pyx":760
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],             # <<<<<<<<<<<<<<
 *                           output.shape[0], output.shape[1]))
 *     if c_median_filter(data.shape[0],   data.shape[1],
 */
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 760; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_5 = 0;
    __pyx_t_3 = 0;

    /* "centrosome/_filter.pyx":759
 *                           mask.shape[0], mask.shape[1]))
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%             # <<<<<<<<<<<<<<
 *                          (data.shape[0],data.shape[1],
 *                           output.shape[0], output.shape[1]))
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Data_shape_d_d_is_not_output_sha, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "centrosome/_filter.pyx":758
 *                          (data.shape[0],data.shape[1],
 *                           mask.shape[0], mask.shape[1]))
 *     if data.shape[0] != output.shape[0] or data.shape[1] != output.shape[1]:             # <<<<<<<<<<<<<<
 *         raise ValueError('Data shape (%d,%d) is not output shape (%d,%d)'%
 *                          (data.shape[0],data.shape[1],
 */
  }

  /* "centrosome/_filter.pyx":762
 *                          (data.shape[0],data.shape[1],
 *                           output.shape[0], output.shape[1]))
 *     if c_median_filter(data.shape[0],   data.shape[1],             # <<<<<<<<<<<<<<
 *                        data.strides[0], data.strides[1],
 *                        radius, percent,
 */
  __pyx_t_1 = (__pyx_f_10centrosome_7_filter_c_median_filter((__pyx_v_data->dimensions[0]), (__pyx_v_data->dimensions[1]), (__pyx_v_data->strides[0]), (__pyx_v_data->strides[1]), __pyx_v_radius, __pyx_v_percent, ((__pyx_t_5numpy_uint8_t *)__pyx_v_data->data), ((__pyx_t_5numpy_uint8_t *)__pyx_v_mask->data), ((__pyx_t_5numpy_uint8_t *)__pyx_v_output->data)) != 0);
  if (__pyx_t_1) {

    /* "centrosome/_filter.pyx":768
 *                        <np.uint8_t *>mask.data,
 *                        <np.uint8_t *>output.data):
 *         raise MemoryError('Failed to allocate scratchpad memory')             # <<<<<<<<<<<<<<
 * 
 * @cython.boundscheck(False)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 768; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 768; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "centrosome/_filter.pyx":762
 *                          (data.shape[0],data.shape[1],
 *                           output.shape[0], output.shape[1]))
 *     if c_median_filter(data.shape[0],   data.shape[1],             # <<<<<<<<<<<<<<
 *                        data.strides[0], data.strides[1],
 *                        radius, percent,
 */
  }

  /* "centrosome/_filter.pyx":732
 *     return 0
 * 
 * def median_filter(np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] output,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_output.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("centrosome._filter.median_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_output.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":771
 * 
 * @cython.boundscheck(False)
 * def masked_convolution(np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                        np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                        np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] kernel):
 */

/* Python wrapper */
static PyObject *__pyx_pw_10centrosome_7_filter_3masked_convolution(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10centrosome_7_filter_2masked_convolution[] = "Convolution respecting a mask\n\n    data - a 2d array containing the image data\n    mask - a mask of relevant points.\n    kernel - a square convolution kernel of odd dimension\n    ";
static PyMethodDef __pyx_mdef_10centrosome_7_filter_3masked_convolution = {"masked_convolution", (PyCFunction)__pyx_pw_10centrosome_7_filter_3masked_convolution, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10centrosome_7_filter_2masked_convolution};
static PyObject *__pyx_pw_10centrosome_7_filter_3masked_convolution(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_data = 0;
  PyArrayObject *__pyx_v_mask = 0;
  PyArrayObject *__pyx_v_kernel = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("masked_convolution (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_mask,&__pyx_n_s_kernel,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("masked_convolution", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kernel)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("masked_convolution", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "masked_convolution") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_data = ((PyArrayObject *)values[0]);
    __pyx_v_mask = ((PyArrayObject *)values[1]);
    __pyx_v_kernel = ((PyArrayObject *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("masked_convolution", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("centrosome._filter.masked_convolution", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), __pyx_ptype_5numpy_ndarray, 1, "data", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_mask), __pyx_ptype_5numpy_ndarray, 1, "mask", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 772; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_kernel), __pyx_ptype_5numpy_ndarray, 1, "kernel", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 773; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_10centrosome_7_filter_2masked_convolution(__pyx_self, __pyx_v_data, __pyx_v_mask, __pyx_v_kernel);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10centrosome_7_filter_2masked_convolution(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_data, PyArrayObject *__pyx_v_mask, PyArrayObject *__pyx_v_kernel) {
  __pyx_t_5numpy_float64_t *__pyx_v_pkernel;
  __pyx_t_5numpy_float64_t *__pyx_v_pimage;
  __pyx_t_5numpy_uint8_t *__pyx_v_pmask;
  __pyx_t_5numpy_float64_t *__pyx_v_poutput;
  int __pyx_v_kernel_width;
  int __pyx_v_kernel_half_width;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_ik;
  int __pyx_v_jk;
  int __pyx_v_istride;
  int __pyx_v_mstride;
  int __pyx_v_kstride;
  __pyx_t_5numpy_float64_t __pyx_v_accumulator;
  PyArrayObject *__pyx_v_big_mask = 0;
  PyArrayObject *__pyx_v_output = 0;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_big_mask;
  __Pyx_Buffer __pyx_pybuffer_big_mask;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_data;
  __Pyx_Buffer __pyx_pybuffer_data;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_kernel;
  __Pyx_Buffer __pyx_pybuffer_kernel;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_mask;
  __Pyx_Buffer __pyx_pybuffer_mask;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_output;
  __Pyx_Buffer __pyx_pybuffer_output;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyArrayObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyArrayObject *__pyx_t_13 = NULL;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  long __pyx_t_18;
  int __pyx_t_19;
  long __pyx_t_20;
  int __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("masked_convolution", 0);
  __pyx_pybuffer_big_mask.pybuffer.buf = NULL;
  __pyx_pybuffer_big_mask.refcount = 0;
  __pyx_pybuffernd_big_mask.data = NULL;
  __pyx_pybuffernd_big_mask.rcbuffer = &__pyx_pybuffer_big_mask;
  __pyx_pybuffer_output.pybuffer.buf = NULL;
  __pyx_pybuffer_output.refcount = 0;
  __pyx_pybuffernd_output.data = NULL;
  __pyx_pybuffernd_output.rcbuffer = &__pyx_pybuffer_output;
  __pyx_pybuffer_data.pybuffer.buf = NULL;
  __pyx_pybuffer_data.refcount = 0;
  __pyx_pybuffernd_data.data = NULL;
  __pyx_pybuffernd_data.rcbuffer = &__pyx_pybuffer_data;
  __pyx_pybuffer_mask.pybuffer.buf = NULL;
  __pyx_pybuffer_mask.refcount = 0;
  __pyx_pybuffernd_mask.data = NULL;
  __pyx_pybuffernd_mask.rcbuffer = &__pyx_pybuffer_mask;
  __pyx_pybuffer_kernel.pybuffer.buf = NULL;
  __pyx_pybuffer_kernel.refcount = 0;
  __pyx_pybuffernd_kernel.data = NULL;
  __pyx_pybuffernd_kernel.rcbuffer = &__pyx_pybuffer_kernel;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_data.rcbuffer->pybuffer, (PyObject*)__pyx_v_data, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_data.diminfo[0].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_data.diminfo[0].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_data.diminfo[1].strides = __pyx_pybuffernd_data.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_data.diminfo[1].shape = __pyx_pybuffernd_data.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_mask.rcbuffer->pybuffer, (PyObject*)__pyx_v_mask, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_mask.diminfo[0].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_mask.diminfo[0].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_mask.diminfo[1].strides = __pyx_pybuffernd_mask.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_mask.diminfo[1].shape = __pyx_pybuffernd_mask.rcbuffer->pybuffer.shape[1];
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_kernel.rcbuffer->pybuffer, (PyObject*)__pyx_v_kernel, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_kernel.diminfo[0].strides = __pyx_pybuffernd_kernel.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_kernel.diminfo[0].shape = __pyx_pybuffernd_kernel.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_kernel.diminfo[1].strides = __pyx_pybuffernd_kernel.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_kernel.diminfo[1].shape = __pyx_pybuffernd_kernel.rcbuffer->pybuffer.shape[1];

  /* "centrosome/_filter.pyx":797
 *         np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] output
 * 
 *     assert kernel.shape[0] % 2 == 1, "Kernel shape must be odd"             # <<<<<<<<<<<<<<
 *     assert kernel.shape[0]==kernel.shape[1], "Kernel must be square"
 *     assert mask.shape[0]==data.shape[0]
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__Pyx_mod_long((__pyx_v_kernel->dimensions[0]), 2) == 1) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Kernel_shape_must_be_odd);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 797; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "centrosome/_filter.pyx":798
 * 
 *     assert kernel.shape[0] % 2 == 1, "Kernel shape must be odd"
 *     assert kernel.shape[0]==kernel.shape[1], "Kernel must be square"             # <<<<<<<<<<<<<<
 *     assert mask.shape[0]==data.shape[0]
 *     assert mask.shape[1]==data.shape[1]
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_kernel->dimensions[0]) == (__pyx_v_kernel->dimensions[1])) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Kernel_must_be_square);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "centrosome/_filter.pyx":799
 *     assert kernel.shape[0] % 2 == 1, "Kernel shape must be odd"
 *     assert kernel.shape[0]==kernel.shape[1], "Kernel must be square"
 *     assert mask.shape[0]==data.shape[0]             # <<<<<<<<<<<<<<
 *     assert mask.shape[1]==data.shape[1]
 *     kernel_width = kernel.shape[0]
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_mask->dimensions[0]) == (__pyx_v_data->dimensions[0])) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "centrosome/_filter.pyx":800
 *     assert kernel.shape[0]==kernel.shape[1], "Kernel must be square"
 *     assert mask.shape[0]==data.shape[0]
 *     assert mask.shape[1]==data.shape[1]             # <<<<<<<<<<<<<<
 *     kernel_width = kernel.shape[0]
 *     kernel_half_width = kernel_width / 2
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(((__pyx_v_mask->dimensions[1]) == (__pyx_v_data->dimensions[1])) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 800; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "centrosome/_filter.pyx":801
 *     assert mask.shape[0]==data.shape[0]
 *     assert mask.shape[1]==data.shape[1]
 *     kernel_width = kernel.shape[0]             # <<<<<<<<<<<<<<
 *     kernel_half_width = kernel_width / 2
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)
 */
  __pyx_v_kernel_width = (__pyx_v_kernel->dimensions[0]);

  /* "centrosome/_filter.pyx":802
 *     assert mask.shape[1]==data.shape[1]
 *     kernel_width = kernel.shape[0]
 *     kernel_half_width = kernel_width / 2             # <<<<<<<<<<<<<<
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)
 */
  __pyx_v_kernel_half_width = __Pyx_div_long(__pyx_v_kernel_width, 2);

  /* "centrosome/_filter.pyx":803
 *     kernel_width = kernel.shape[0]
 *     kernel_half_width = kernel_width / 2
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)             # <<<<<<<<<<<<<<
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)
 *     big_mask[kernel_half_width:kernel_half_width+data.shape[0],
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(((__pyx_v_data->dimensions[0]) + __pyx_v_kernel_width)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(((__pyx_v_data->dimensions[1]) + __pyx_v_kernel_width)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __pyx_t_2 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_uint8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_6 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_2);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_big_mask.rcbuffer->pybuffer);
    __pyx_t_9 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_big_mask.rcbuffer->pybuffer, (PyObject*)__pyx_t_8, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_9 < 0)) {
      PyErr_Fetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_big_mask.rcbuffer->pybuffer, (PyObject*)__pyx_v_big_mask, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_10); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_12);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_10, __pyx_t_11, __pyx_t_12);
      }
    }
    __pyx_pybuffernd_big_mask.diminfo[0].strides = __pyx_pybuffernd_big_mask.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_big_mask.diminfo[0].shape = __pyx_pybuffernd_big_mask.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_big_mask.diminfo[1].strides = __pyx_pybuffernd_big_mask.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_big_mask.diminfo[1].shape = __pyx_pybuffernd_big_mask.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_8 = 0;
  __pyx_v_big_mask = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":804
 *     kernel_half_width = kernel_width / 2
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)             # <<<<<<<<<<<<<<
 *     big_mask[kernel_half_width:kernel_half_width+data.shape[0],
 *              kernel_half_width:kernel_half_width+data.shape[1]] = mask
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_data->dimensions[1])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_data), __pyx_n_s_dtype); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_6 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_t_2);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_13 = ((PyArrayObject *)__pyx_t_1);
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_output.rcbuffer->pybuffer);
    __pyx_t_9 = __Pyx_GetBufferAndValidate(&__pyx_pybuffernd_output.rcbuffer->pybuffer, (PyObject*)__pyx_t_13, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack);
    if (unlikely(__pyx_t_9 < 0)) {
      PyErr_Fetch(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);
      if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_output.rcbuffer->pybuffer, (PyObject*)__pyx_v_output, &__Pyx_TypeInfo_nn___pyx_t_5numpy_float64_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 2, 0, __pyx_stack) == -1)) {
        Py_XDECREF(__pyx_t_12); Py_XDECREF(__pyx_t_11); Py_XDECREF(__pyx_t_10);
        __Pyx_RaiseBufferFallbackError();
      } else {
        PyErr_Restore(__pyx_t_12, __pyx_t_11, __pyx_t_10);
      }
    }
    __pyx_pybuffernd_output.diminfo[0].strides = __pyx_pybuffernd_output.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_output.diminfo[0].shape = __pyx_pybuffernd_output.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_output.diminfo[1].strides = __pyx_pybuffernd_output.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_output.diminfo[1].shape = __pyx_pybuffernd_output.rcbuffer->pybuffer.shape[1];
    if (unlikely(__pyx_t_9 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 804; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_13 = 0;
  __pyx_v_output = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":805
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)
 *     big_mask[kernel_half_width:kernel_half_width+data.shape[0],             # <<<<<<<<<<<<<<
 *              kernel_half_width:kernel_half_width+data.shape[1]] = mask
 *     #
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_kernel_half_width); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_kernel_half_width + (__pyx_v_data->dimensions[0]))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = PySlice_New(__pyx_t_1, __pyx_t_7, Py_None); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "centrosome/_filter.pyx":806
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)
 *     big_mask[kernel_half_width:kernel_half_width+data.shape[0],
 *              kernel_half_width:kernel_half_width+data.shape[1]] = mask             # <<<<<<<<<<<<<<
 *     #
 *     # stride in number of elements across the i direction
 */
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_kernel_half_width); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 806; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_kernel_half_width + (__pyx_v_data->dimensions[1]))); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 806; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);

  /* "centrosome/_filter.pyx":805
 *     big_mask = np.zeros((data.shape[0]+kernel_width, data.shape[1]+kernel_width), np.uint8)
 *     output   = np.zeros((data.shape[0],data.shape[1]), data.dtype)
 *     big_mask[kernel_half_width:kernel_half_width+data.shape[0],             # <<<<<<<<<<<<<<
 *              kernel_half_width:kernel_half_width+data.shape[1]] = mask
 *     #
 */
  __pyx_t_2 = PySlice_New(__pyx_t_7, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  if (unlikely(PyObject_SetItem(((PyObject *)__pyx_v_big_mask), __pyx_t_1, ((PyObject *)__pyx_v_mask)) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":810
 *     # stride in number of elements across the i direction
 *     #
 *     istride = data.strides[0] / PyArray_ITEMSIZE(data)             # <<<<<<<<<<<<<<
 *     mstride = big_mask.strides[0] / PyArray_ITEMSIZE(big_mask)
 *     kstride = kernel.strides[0] / PyArray_ITEMSIZE(kernel)
 */
  __pyx_t_9 = PyArray_ITEMSIZE(((PyArrayObject *)__pyx_v_data));
  if (unlikely(__pyx_t_9 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  else if (sizeof(npy_intp) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_t_9 == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW((__pyx_v_data->strides[0])))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_istride = __Pyx_div_npy_intp((__pyx_v_data->strides[0]), __pyx_t_9);

  /* "centrosome/_filter.pyx":811
 *     #
 *     istride = data.strides[0] / PyArray_ITEMSIZE(data)
 *     mstride = big_mask.strides[0] / PyArray_ITEMSIZE(big_mask)             # <<<<<<<<<<<<<<
 *     kstride = kernel.strides[0] / PyArray_ITEMSIZE(kernel)
 *     #
 */
  __pyx_t_9 = PyArray_ITEMSIZE(((PyArrayObject *)__pyx_v_big_mask));
  if (unlikely(__pyx_t_9 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  else if (sizeof(npy_intp) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_t_9 == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW((__pyx_v_big_mask->strides[0])))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_mstride = __Pyx_div_npy_intp((__pyx_v_big_mask->strides[0]), __pyx_t_9);

  /* "centrosome/_filter.pyx":812
 *     istride = data.strides[0] / PyArray_ITEMSIZE(data)
 *     mstride = big_mask.strides[0] / PyArray_ITEMSIZE(big_mask)
 *     kstride = kernel.strides[0] / PyArray_ITEMSIZE(kernel)             # <<<<<<<<<<<<<<
 *     #
 *     # pointers to data. pmask is offset to point at the 0,0 element
 */
  __pyx_t_9 = PyArray_ITEMSIZE(((PyArrayObject *)__pyx_v_kernel));
  if (unlikely(__pyx_t_9 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 812; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  else if (sizeof(npy_intp) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_t_9 == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW((__pyx_v_kernel->strides[0])))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 812; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_kstride = __Pyx_div_npy_intp((__pyx_v_kernel->strides[0]), __pyx_t_9);

  /* "centrosome/_filter.pyx":817
 *     # pkernel is offset to point at the middle of the kernel
 *     #
 *     pmask   = <np.uint8_t *>(big_mask.data + kernel_half_width *             # <<<<<<<<<<<<<<
 *                              (big_mask.strides[0] + big_mask.strides[1]))
 *     pimage  = <np.float64_t *>(data.data)
 */
  __pyx_v_pmask = ((__pyx_t_5numpy_uint8_t *)(__pyx_v_big_mask->data + (__pyx_v_kernel_half_width * ((__pyx_v_big_mask->strides[0]) + (__pyx_v_big_mask->strides[1])))));

  /* "centrosome/_filter.pyx":819
 *     pmask   = <np.uint8_t *>(big_mask.data + kernel_half_width *
 *                              (big_mask.strides[0] + big_mask.strides[1]))
 *     pimage  = <np.float64_t *>(data.data)             # <<<<<<<<<<<<<<
 *     pkernel = <np.float64_t *>(kernel.data)+(kstride+1) * kernel_half_width
 *     poutput = <np.float64_t *>(output.data)
 */
  __pyx_v_pimage = ((__pyx_t_5numpy_float64_t *)__pyx_v_data->data);

  /* "centrosome/_filter.pyx":820
 *                              (big_mask.strides[0] + big_mask.strides[1]))
 *     pimage  = <np.float64_t *>(data.data)
 *     pkernel = <np.float64_t *>(kernel.data)+(kstride+1) * kernel_half_width             # <<<<<<<<<<<<<<
 *     poutput = <np.float64_t *>(output.data)
 *     for i in range(data.shape[0]):
 */
  __pyx_v_pkernel = (((__pyx_t_5numpy_float64_t *)__pyx_v_kernel->data) + ((__pyx_v_kstride + 1) * __pyx_v_kernel_half_width));

  /* "centrosome/_filter.pyx":821
 *     pimage  = <np.float64_t *>(data.data)
 *     pkernel = <np.float64_t *>(kernel.data)+(kstride+1) * kernel_half_width
 *     poutput = <np.float64_t *>(output.data)             # <<<<<<<<<<<<<<
 *     for i in range(data.shape[0]):
 *         for j in range(data.shape[1]):
 */
  __pyx_v_poutput = ((__pyx_t_5numpy_float64_t *)__pyx_v_output->data);

  /* "centrosome/_filter.pyx":822
 *     pkernel = <np.float64_t *>(kernel.data)+(kstride+1) * kernel_half_width
 *     poutput = <np.float64_t *>(output.data)
 *     for i in range(data.shape[0]):             # <<<<<<<<<<<<<<
 *         for j in range(data.shape[1]):
 *             if pmask[i*mstride+j] == 0:
 */
  __pyx_t_14 = (__pyx_v_data->dimensions[0]);
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_14; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "centrosome/_filter.pyx":823
 *     poutput = <np.float64_t *>(output.data)
 *     for i in range(data.shape[0]):
 *         for j in range(data.shape[1]):             # <<<<<<<<<<<<<<
 *             if pmask[i*mstride+j] == 0:
 *                 continue
 */
    __pyx_t_15 = (__pyx_v_data->dimensions[1]);
    for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
      __pyx_v_j = __pyx_t_16;

      /* "centrosome/_filter.pyx":824
 *     for i in range(data.shape[0]):
 *         for j in range(data.shape[1]):
 *             if pmask[i*mstride+j] == 0:             # <<<<<<<<<<<<<<
 *                 continue
 *             accumulator = 0
 */
      __pyx_t_17 = (((__pyx_v_pmask[((__pyx_v_i * __pyx_v_mstride) + __pyx_v_j)]) == 0) != 0);
      if (__pyx_t_17) {

        /* "centrosome/_filter.pyx":825
 *         for j in range(data.shape[1]):
 *             if pmask[i*mstride+j] == 0:
 *                 continue             # <<<<<<<<<<<<<<
 *             accumulator = 0
 *             for ik in range(-kernel_half_width,kernel_half_width+1):
 */
        goto __pyx_L5_continue;

        /* "centrosome/_filter.pyx":824
 *     for i in range(data.shape[0]):
 *         for j in range(data.shape[1]):
 *             if pmask[i*mstride+j] == 0:             # <<<<<<<<<<<<<<
 *                 continue
 *             accumulator = 0
 */
      }

      /* "centrosome/_filter.pyx":826
 *             if pmask[i*mstride+j] == 0:
 *                 continue
 *             accumulator = 0             # <<<<<<<<<<<<<<
 *             for ik in range(-kernel_half_width,kernel_half_width+1):
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):
 */
      __pyx_v_accumulator = 0.0;

      /* "centrosome/_filter.pyx":827
 *                 continue
 *             accumulator = 0
 *             for ik in range(-kernel_half_width,kernel_half_width+1):             # <<<<<<<<<<<<<<
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):
 *                     if pmask[(i+ik)*mstride+j+jk] != 0:
 */
      __pyx_t_18 = (__pyx_v_kernel_half_width + 1);
      for (__pyx_t_19 = (-__pyx_v_kernel_half_width); __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
        __pyx_v_ik = __pyx_t_19;

        /* "centrosome/_filter.pyx":828
 *             accumulator = 0
 *             for ik in range(-kernel_half_width,kernel_half_width+1):
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):             # <<<<<<<<<<<<<<
 *                     if pmask[(i+ik)*mstride+j+jk] != 0:
 *                         accumulator += (pkernel[ik*kstride+jk] *
 */
        __pyx_t_20 = (__pyx_v_kernel_half_width + 1);
        for (__pyx_t_21 = (-__pyx_v_kernel_half_width); __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
          __pyx_v_jk = __pyx_t_21;

          /* "centrosome/_filter.pyx":829
 *             for ik in range(-kernel_half_width,kernel_half_width+1):
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):
 *                     if pmask[(i+ik)*mstride+j+jk] != 0:             # <<<<<<<<<<<<<<
 *                         accumulator += (pkernel[ik*kstride+jk] *
 *                                         pimage[(i+ik)*istride+j+jk])
 */
          __pyx_t_17 = (((__pyx_v_pmask[((((__pyx_v_i + __pyx_v_ik) * __pyx_v_mstride) + __pyx_v_j) + __pyx_v_jk)]) != 0) != 0);
          if (__pyx_t_17) {

            /* "centrosome/_filter.pyx":830
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):
 *                     if pmask[(i+ik)*mstride+j+jk] != 0:
 *                         accumulator += (pkernel[ik*kstride+jk] *             # <<<<<<<<<<<<<<
 *                                         pimage[(i+ik)*istride+j+jk])
 *             poutput[i*istride+j] = accumulator
 */
            __pyx_v_accumulator = (__pyx_v_accumulator + ((__pyx_v_pkernel[((__pyx_v_ik * __pyx_v_kstride) + __pyx_v_jk)]) * (__pyx_v_pimage[((((__pyx_v_i + __pyx_v_ik) * __pyx_v_istride) + __pyx_v_j) + __pyx_v_jk)])));

            /* "centrosome/_filter.pyx":829
 *             for ik in range(-kernel_half_width,kernel_half_width+1):
 *                 for jk in range(-kernel_half_width,kernel_half_width+1):
 *                     if pmask[(i+ik)*mstride+j+jk] != 0:             # <<<<<<<<<<<<<<
 *                         accumulator += (pkernel[ik*kstride+jk] *
 *                                         pimage[(i+ik)*istride+j+jk])
 */
          }
        }
      }

      /* "centrosome/_filter.pyx":832
 *                         accumulator += (pkernel[ik*kstride+jk] *
 *                                         pimage[(i+ik)*istride+j+jk])
 *             poutput[i*istride+j] = accumulator             # <<<<<<<<<<<<<<
 *     return output
 * 
 */
      (__pyx_v_poutput[((__pyx_v_i * __pyx_v_istride) + __pyx_v_j)]) = __pyx_v_accumulator;
      __pyx_L5_continue:;
    }
  }

  /* "centrosome/_filter.pyx":833
 *                                         pimage[(i+ik)*istride+j+jk])
 *             poutput[i*istride+j] = accumulator
 *     return output             # <<<<<<<<<<<<<<
 * 
 * @cython.boundscheck(False)
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_output));
  __pyx_r = ((PyObject *)__pyx_v_output);
  goto __pyx_L0;

  /* "centrosome/_filter.pyx":771
 * 
 * @cython.boundscheck(False)
 * def masked_convolution(np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                        np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                        np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] kernel):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_big_mask.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_kernel.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_output.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("centrosome._filter.masked_convolution", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_big_mask.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_data.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_kernel.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_mask.rcbuffer->pybuffer);
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_output.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_big_mask);
  __Pyx_XDECREF((PyObject *)__pyx_v_output);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "centrosome/_filter.pyx":837
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * def paeth_decoder(             # <<<<<<<<<<<<<<
 *     np.ndarray[dtype=np.uint8_t, ndim=3, negative_indices=False, mode='c'] x,
 *     np.int32_t raster_count):
 */

/* Python wrapper */
static PyObject *__pyx_pw_10centrosome_7_filter_5paeth_decoder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_10centrosome_7_filter_4paeth_decoder[] = "Paeth decoder - reverse Paeth filter\n    \n    x: matrix of bytes. The first dimension indexes rasters. The second\n       dimension indexes pixel positions (stride = 24 for interleaved color,\n       = 8 for monochrome). The third dimension indexes the bytes within\n       the pixel. If your image consists of multiple planes, you should\n       combine the planar dimensions on input by changing the image shape.\n       \n    raster_count: # of rasters in a plane\n    \n    Given a 2-dimensional array of unsigned bytes, the Paeth filter\n    looks at 4 elements\n    \n    C B\n    A x\n    \n    p = A+B-C\n    estimate = A if abs(p-A) <= abs(p-B) and similarly for C\n             = B if abs(p-B) <= abs(p-C)\n             = C otherwise\n    x += estimate if reverse\n    \n    Citation: http://www.w3.org/TR/PNG-Filters.html\n    ";
static PyMethodDef __pyx_mdef_10centrosome_7_filter_5paeth_decoder = {"paeth_decoder", (PyCFunction)__pyx_pw_10centrosome_7_filter_5paeth_decoder, METH_VARARGS|METH_KEYWORDS, __pyx_doc_10centrosome_7_filter_4paeth_decoder};
static PyObject *__pyx_pw_10centrosome_7_filter_5paeth_decoder(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyArrayObject *__pyx_v_x = 0;
  __pyx_t_5numpy_int32_t __pyx_v_raster_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("paeth_decoder (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_raster_count,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_raster_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("paeth_decoder", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "paeth_decoder") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = ((PyArrayObject *)values[0]);
    __pyx_v_raster_count = __Pyx_PyInt_As_npy_int32(values[1]); if (unlikely((__pyx_v_raster_count == (npy_int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("paeth_decoder", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("centrosome._filter.paeth_decoder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_5numpy_ndarray, 1, "x", 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_10centrosome_7_filter_4paeth_decoder(__pyx_self, __pyx_v_x, __pyx_v_raster_count);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10centrosome_7_filter_4paeth_decoder(CYTHON_UNUSED PyObject *__pyx_self, PyArrayObject *__pyx_v_x, __pyx_t_5numpy_int32_t __pyx_v_raster_count) {
  __pyx_t_5numpy_int32_t __pyx_v_raster_stride;
  __pyx_t_5numpy_int32_t __pyx_v_pixel_stride;
  unsigned char *__pyx_v_ptr;
  __pyx_t_5numpy_int32_t __pyx_v_a;
  __pyx_t_5numpy_int32_t __pyx_v_b;
  __pyx_t_5numpy_int32_t __pyx_v_c;
  __pyx_t_5numpy_int32_t __pyx_v_p;
  __pyx_t_5numpy_int32_t __pyx_v_pa;
  __pyx_t_5numpy_int32_t __pyx_v_pb;
  __pyx_t_5numpy_int32_t __pyx_v_pc;
  __pyx_t_5numpy_int32_t __pyx_v_estimate;
  __pyx_t_5numpy_int32_t __pyx_v_i;
  __pyx_t_5numpy_int32_t __pyx_v_j;
  CYTHON_UNUSED __pyx_t_5numpy_int32_t __pyx_v_k;
  __pyx_t_5numpy_int32_t __pyx_v_imax;
  __pyx_t_5numpy_int32_t __pyx_v_jmax;
  __pyx_t_5numpy_int32_t __pyx_v_kmax;
  __pyx_t_5numpy_int32_t __pyx_v_raster_number;
  __Pyx_LocalBuf_ND __pyx_pybuffernd_x;
  __Pyx_Buffer __pyx_pybuffer_x;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_5numpy_int32_t __pyx_t_1;
  __pyx_t_5numpy_int32_t __pyx_t_2;
  __pyx_t_5numpy_int32_t __pyx_t_3;
  int __pyx_t_4;
  __pyx_t_5numpy_int32_t __pyx_t_5;
  int __pyx_t_6;
  long __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("paeth_decoder", 0);
  __pyx_pybuffer_x.pybuffer.buf = NULL;
  __pyx_pybuffer_x.refcount = 0;
  __pyx_pybuffernd_x.data = NULL;
  __pyx_pybuffernd_x.rcbuffer = &__pyx_pybuffer_x;
  {
    __Pyx_BufFmt_StackElem __pyx_stack[1];
    if (unlikely(__Pyx_GetBufferAndValidate(&__pyx_pybuffernd_x.rcbuffer->pybuffer, (PyObject*)__pyx_v_x, &__Pyx_TypeInfo_nn___pyx_t_5numpy_uint8_t, PyBUF_FORMAT| PyBUF_C_CONTIGUOUS, 3, 0, __pyx_stack) == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_pybuffernd_x.diminfo[0].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[0]; __pyx_pybuffernd_x.diminfo[0].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[0]; __pyx_pybuffernd_x.diminfo[1].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[1]; __pyx_pybuffernd_x.diminfo[1].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[1]; __pyx_pybuffernd_x.diminfo[2].strides = __pyx_pybuffernd_x.rcbuffer->pybuffer.strides[2]; __pyx_pybuffernd_x.diminfo[2].shape = __pyx_pybuffernd_x.rcbuffer->pybuffer.shape[2];

  /* "centrosome/_filter.pyx":865
 *     '''
 *     cdef:
 *         np.int32_t raster_stride = x.strides[0]             # <<<<<<<<<<<<<<
 *         np.int32_t pixel_stride = x.strides[1]
 *         unsigned char *ptr = <unsigned char *>x.data
 */
  __pyx_v_raster_stride = (__pyx_v_x->strides[0]);

  /* "centrosome/_filter.pyx":866
 *     cdef:
 *         np.int32_t raster_stride = x.strides[0]
 *         np.int32_t pixel_stride = x.strides[1]             # <<<<<<<<<<<<<<
 *         unsigned char *ptr = <unsigned char *>x.data
 *         np.int32_t a,b,c,p,pa,pb,pc,estimate
 */
  __pyx_v_pixel_stride = (__pyx_v_x->strides[1]);

  /* "centrosome/_filter.pyx":867
 *         np.int32_t raster_stride = x.strides[0]
 *         np.int32_t pixel_stride = x.strides[1]
 *         unsigned char *ptr = <unsigned char *>x.data             # <<<<<<<<<<<<<<
 *         np.int32_t a,b,c,p,pa,pb,pc,estimate
 *         np.int32_t i,j,k
 */
  __pyx_v_ptr = ((unsigned char *)__pyx_v_x->data);

  /* "centrosome/_filter.pyx":870
 *         np.int32_t a,b,c,p,pa,pb,pc,estimate
 *         np.int32_t i,j,k
 *         np.int32_t imax = x.shape[0]             # <<<<<<<<<<<<<<
 *         np.int32_t jmax = x.shape[1]
 *         np.int32_t kmax = x.shape[2]
 */
  __pyx_v_imax = (__pyx_v_x->dimensions[0]);

  /* "centrosome/_filter.pyx":871
 *         np.int32_t i,j,k
 *         np.int32_t imax = x.shape[0]
 *         np.int32_t jmax = x.shape[1]             # <<<<<<<<<<<<<<
 *         np.int32_t kmax = x.shape[2]
 *         np.int32_t raster_number
 */
  __pyx_v_jmax = (__pyx_v_x->dimensions[1]);

  /* "centrosome/_filter.pyx":872
 *         np.int32_t imax = x.shape[0]
 *         np.int32_t jmax = x.shape[1]
 *         np.int32_t kmax = x.shape[2]             # <<<<<<<<<<<<<<
 *         np.int32_t raster_number
 *         np.int32_t plane_number
 */
  __pyx_v_kmax = (__pyx_v_x->dimensions[2]);

  /* "centrosome/_filter.pyx":876
 *         np.int32_t plane_number
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i from 0<=i<imax:
 *             raster_number = i % raster_count
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "centrosome/_filter.pyx":877
 * 
 *     with nogil:
 *         for i from 0<=i<imax:             # <<<<<<<<<<<<<<
 *             raster_number = i % raster_count
 *             for j from 0<=j<jmax:
 */
        __pyx_t_1 = __pyx_v_imax;
        for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

          /* "centrosome/_filter.pyx":878
 *     with nogil:
 *         for i from 0<=i<imax:
 *             raster_number = i % raster_count             # <<<<<<<<<<<<<<
 *             for j from 0<=j<jmax:
 *                 for k from 0<=k<kmax:
 */
          __pyx_v_raster_number = (__pyx_v_i % __pyx_v_raster_count);

          /* "centrosome/_filter.pyx":879
 *         for i from 0<=i<imax:
 *             raster_number = i % raster_count
 *             for j from 0<=j<jmax:             # <<<<<<<<<<<<<<
 *                 for k from 0<=k<kmax:
 *                     if raster_number == 0:
 */
          __pyx_t_2 = __pyx_v_jmax;
          for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_2; __pyx_v_j++) {

            /* "centrosome/_filter.pyx":880
 *             raster_number = i % raster_count
 *             for j from 0<=j<jmax:
 *                 for k from 0<=k<kmax:             # <<<<<<<<<<<<<<
 *                     if raster_number == 0:
 *                         b = c = 0
 */
            __pyx_t_3 = __pyx_v_kmax;
            for (__pyx_v_k = 0; __pyx_v_k < __pyx_t_3; __pyx_v_k++) {

              /* "centrosome/_filter.pyx":881
 *             for j from 0<=j<jmax:
 *                 for k from 0<=k<kmax:
 *                     if raster_number == 0:             # <<<<<<<<<<<<<<
 *                         b = c = 0
 *                         if j==0:
 */
              __pyx_t_4 = ((__pyx_v_raster_number == 0) != 0);
              if (__pyx_t_4) {

                /* "centrosome/_filter.pyx":882
 *                 for k from 0<=k<kmax:
 *                     if raster_number == 0:
 *                         b = c = 0             # <<<<<<<<<<<<<<
 *                         if j==0:
 *                             a = 0
 */
                __pyx_v_b = 0;
                __pyx_v_c = 0;

                /* "centrosome/_filter.pyx":883
 *                     if raster_number == 0:
 *                         b = c = 0
 *                         if j==0:             # <<<<<<<<<<<<<<
 *                             a = 0
 *                         else:
 */
                __pyx_t_4 = ((__pyx_v_j == 0) != 0);
                if (__pyx_t_4) {

                  /* "centrosome/_filter.pyx":884
 *                         b = c = 0
 *                         if j==0:
 *                             a = 0             # <<<<<<<<<<<<<<
 *                         else:
 *                             a = ptr[-pixel_stride]
 */
                  __pyx_v_a = 0;

                  /* "centrosome/_filter.pyx":883
 *                     if raster_number == 0:
 *                         b = c = 0
 *                         if j==0:             # <<<<<<<<<<<<<<
 *                             a = 0
 *                         else:
 */
                  goto __pyx_L13;
                }

                /* "centrosome/_filter.pyx":886
 *                             a = 0
 *                         else:
 *                             a = ptr[-pixel_stride]             # <<<<<<<<<<<<<<
 *                     else:
 *                         b = ptr[-raster_stride]
 */
                /*else*/ {
                  __pyx_v_a = (__pyx_v_ptr[(-__pyx_v_pixel_stride)]);
                }
                __pyx_L13:;

                /* "centrosome/_filter.pyx":881
 *             for j from 0<=j<jmax:
 *                 for k from 0<=k<kmax:
 *                     if raster_number == 0:             # <<<<<<<<<<<<<<
 *                         b = c = 0
 *                         if j==0:
 */
                goto __pyx_L12;
              }

              /* "centrosome/_filter.pyx":888
 *                             a = ptr[-pixel_stride]
 *                     else:
 *                         b = ptr[-raster_stride]             # <<<<<<<<<<<<<<
 *                         if j==0:
 *                            a = c = 0
 */
              /*else*/ {
                __pyx_v_b = (__pyx_v_ptr[(-__pyx_v_raster_stride)]);

                /* "centrosome/_filter.pyx":889
 *                     else:
 *                         b = ptr[-raster_stride]
 *                         if j==0:             # <<<<<<<<<<<<<<
 *                            a = c = 0
 *                         else:
 */
                __pyx_t_4 = ((__pyx_v_j == 0) != 0);
                if (__pyx_t_4) {

                  /* "centrosome/_filter.pyx":890
 *                         b = ptr[-raster_stride]
 *                         if j==0:
 *                            a = c = 0             # <<<<<<<<<<<<<<
 *                         else:
 *                            a = ptr[-pixel_stride]
 */
                  __pyx_v_a = 0;
                  __pyx_v_c = 0;

                  /* "centrosome/_filter.pyx":889
 *                     else:
 *                         b = ptr[-raster_stride]
 *                         if j==0:             # <<<<<<<<<<<<<<
 *                            a = c = 0
 *                         else:
 */
                  goto __pyx_L14;
                }

                /* "centrosome/_filter.pyx":892
 *                            a = c = 0
 *                         else:
 *                            a = ptr[-pixel_stride]             # <<<<<<<<<<<<<<
 *                            c = ptr[-raster_stride-pixel_stride]
 *                     p = a + b - c
 */
                /*else*/ {
                  __pyx_v_a = (__pyx_v_ptr[(-__pyx_v_pixel_stride)]);

                  /* "centrosome/_filter.pyx":893
 *                         else:
 *                            a = ptr[-pixel_stride]
 *                            c = ptr[-raster_stride-pixel_stride]             # <<<<<<<<<<<<<<
 *                     p = a + b - c
 *                     pa = (a-p) if (a>p) else (p-a)
 */
                  __pyx_v_c = (__pyx_v_ptr[((-__pyx_v_raster_stride) - __pyx_v_pixel_stride)]);
                }
                __pyx_L14:;
              }
              __pyx_L12:;

              /* "centrosome/_filter.pyx":894
 *                            a = ptr[-pixel_stride]
 *                            c = ptr[-raster_stride-pixel_stride]
 *                     p = a + b - c             # <<<<<<<<<<<<<<
 *                     pa = (a-p) if (a>p) else (p-a)
 *                     pb = (b-p) if (b>p) else (p-b)
 */
              __pyx_v_p = ((__pyx_v_a + __pyx_v_b) - __pyx_v_c);

              /* "centrosome/_filter.pyx":895
 *                            c = ptr[-raster_stride-pixel_stride]
 *                     p = a + b - c
 *                     pa = (a-p) if (a>p) else (p-a)             # <<<<<<<<<<<<<<
 *                     pb = (b-p) if (b>p) else (p-b)
 *                     pc = (c-p) if (c>p) else (p-c)
 */
              if (((__pyx_v_a > __pyx_v_p) != 0)) {
                __pyx_t_5 = (__pyx_v_a - __pyx_v_p);
              } else {
                __pyx_t_5 = (__pyx_v_p - __pyx_v_a);
              }
              __pyx_v_pa = __pyx_t_5;

              /* "centrosome/_filter.pyx":896
 *                     p = a + b - c
 *                     pa = (a-p) if (a>p) else (p-a)
 *                     pb = (b-p) if (b>p) else (p-b)             # <<<<<<<<<<<<<<
 *                     pc = (c-p) if (c>p) else (p-c)
 *                     if (pa <= pb) and (pa <= pc):
 */
              if (((__pyx_v_b > __pyx_v_p) != 0)) {
                __pyx_t_5 = (__pyx_v_b - __pyx_v_p);
              } else {
                __pyx_t_5 = (__pyx_v_p - __pyx_v_b);
              }
              __pyx_v_pb = __pyx_t_5;

              /* "centrosome/_filter.pyx":897
 *                     pa = (a-p) if (a>p) else (p-a)
 *                     pb = (b-p) if (b>p) else (p-b)
 *                     pc = (c-p) if (c>p) else (p-c)             # <<<<<<<<<<<<<<
 *                     if (pa <= pb) and (pa <= pc):
 *                         estimate = a
 */
              if (((__pyx_v_c > __pyx_v_p) != 0)) {
                __pyx_t_5 = (__pyx_v_c - __pyx_v_p);
              } else {
                __pyx_t_5 = (__pyx_v_p - __pyx_v_c);
              }
              __pyx_v_pc = __pyx_t_5;

              /* "centrosome/_filter.pyx":898
 *                     pb = (b-p) if (b>p) else (p-b)
 *                     pc = (c-p) if (c>p) else (p-c)
 *                     if (pa <= pb) and (pa <= pc):             # <<<<<<<<<<<<<<
 *                         estimate = a
 *                     elif (pb <= pc):
 */
              __pyx_t_6 = ((__pyx_v_pa <= __pyx_v_pb) != 0);
              if (__pyx_t_6) {
              } else {
                __pyx_t_4 = __pyx_t_6;
                goto __pyx_L16_bool_binop_done;
              }
              __pyx_t_6 = ((__pyx_v_pa <= __pyx_v_pc) != 0);
              __pyx_t_4 = __pyx_t_6;
              __pyx_L16_bool_binop_done:;
              if (__pyx_t_4) {

                /* "centrosome/_filter.pyx":899
 *                     pc = (c-p) if (c>p) else (p-c)
 *                     if (pa <= pb) and (pa <= pc):
 *                         estimate = a             # <<<<<<<<<<<<<<
 *                     elif (pb <= pc):
 *                         estimate = b
 */
                __pyx_v_estimate = __pyx_v_a;

                /* "centrosome/_filter.pyx":898
 *                     pb = (b-p) if (b>p) else (p-b)
 *                     pc = (c-p) if (c>p) else (p-c)
 *                     if (pa <= pb) and (pa <= pc):             # <<<<<<<<<<<<<<
 *                         estimate = a
 *                     elif (pb <= pc):
 */
                goto __pyx_L15;
              }

              /* "centrosome/_filter.pyx":900
 *                     if (pa <= pb) and (pa <= pc):
 *                         estimate = a
 *                     elif (pb <= pc):             # <<<<<<<<<<<<<<
 *                         estimate = b
 *                     else:
 */
              __pyx_t_4 = ((__pyx_v_pb <= __pyx_v_pc) != 0);
              if (__pyx_t_4) {

                /* "centrosome/_filter.pyx":901
 *                         estimate = a
 *                     elif (pb <= pc):
 *                         estimate = b             # <<<<<<<<<<<<<<
 *                     else:
 *                         estimate = c
 */
                __pyx_v_estimate = __pyx_v_b;

                /* "centrosome/_filter.pyx":900
 *                     if (pa <= pb) and (pa <= pc):
 *                         estimate = a
 *                     elif (pb <= pc):             # <<<<<<<<<<<<<<
 *                         estimate = b
 *                     else:
 */
                goto __pyx_L15;
              }

              /* "centrosome/_filter.pyx":903
 *                         estimate = b
 *                     else:
 *                         estimate = c             # <<<<<<<<<<<<<<
 *                     ptr[0] += estimate
 *                     ptr += 1
 */
              /*else*/ {
                __pyx_v_estimate = __pyx_v_c;
              }
              __pyx_L15:;

              /* "centrosome/_filter.pyx":904
 *                     else:
 *                         estimate = c
 *                     ptr[0] += estimate             # <<<<<<<<<<<<<<
 *                     ptr += 1
 */
              __pyx_t_7 = 0;
              (__pyx_v_ptr[__pyx_t_7]) = ((__pyx_v_ptr[__pyx_t_7]) + __pyx_v_estimate);

              /* "centrosome/_filter.pyx":905
 *                         estimate = c
 *                     ptr[0] += estimate
 *                     ptr += 1             # <<<<<<<<<<<<<<
 */
              __pyx_v_ptr = (__pyx_v_ptr + 1);
            }
          }
        }
      }

      /* "centrosome/_filter.pyx":876
 *         np.int32_t plane_number
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i from 0<=i<imax:
 *             raster_number = i % raster_count
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "centrosome/_filter.pyx":837
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * def paeth_decoder(             # <<<<<<<<<<<<<<
 *     np.ndarray[dtype=np.uint8_t, ndim=3, negative_indices=False, mode='c'] x,
 *     np.int32_t raster_count):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  { PyObject *__pyx_type, *__pyx_value, *__pyx_tb;
    __Pyx_ErrFetch(&__pyx_type, &__pyx_value, &__pyx_tb);
    __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);
  __Pyx_ErrRestore(__pyx_type, __pyx_value, __pyx_tb);}
  __Pyx_AddTraceback("centrosome._filter.paeth_decoder", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  goto __pyx_L2;
  __pyx_L0:;
  __Pyx_SafeReleaseBuffer(&__pyx_pybuffernd_x.rcbuffer->pybuffer);
  __pyx_L2:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_filter",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_DTYPE_BOOL, __pyx_k_DTYPE_BOOL, sizeof(__pyx_k_DTYPE_BOOL), 0, 0, 1, 1},
  {&__pyx_n_s_DTYPE_UINT32, __pyx_k_DTYPE_UINT32, sizeof(__pyx_k_DTYPE_UINT32), 0, 0, 1, 1},
  {&__pyx_kp_s_Data_shape_d_d_is_not_mask_shape, __pyx_k_Data_shape_d_d_is_not_mask_shape, sizeof(__pyx_k_Data_shape_d_d_is_not_mask_shape), 0, 0, 1, 0},
  {&__pyx_kp_s_Data_shape_d_d_is_not_output_sha, __pyx_k_Data_shape_d_d_is_not_output_sha, sizeof(__pyx_k_Data_shape_d_d_is_not_output_sha), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_allocate_scratchpad_me, __pyx_k_Failed_to_allocate_scratchpad_me, sizeof(__pyx_k_Failed_to_allocate_scratchpad_me), 0, 0, 1, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_s_Kernel_must_be_square, __pyx_k_Kernel_must_be_square, sizeof(__pyx_k_Kernel_must_be_square), 0, 0, 1, 0},
  {&__pyx_kp_s_Kernel_shape_must_be_odd, __pyx_k_Kernel_shape_must_be_odd, sizeof(__pyx_k_Kernel_shape_must_be_odd), 0, 0, 1, 0},
  {&__pyx_kp_s_Median_filter_percent_d_is_great, __pyx_k_Median_filter_percent_d_is_great, sizeof(__pyx_k_Median_filter_percent_d_is_great), 0, 0, 1, 0},
  {&__pyx_kp_s_Median_filter_percent_d_is_less, __pyx_k_Median_filter_percent_d_is_less, sizeof(__pyx_k_Median_filter_percent_d_is_less), 0, 0, 1, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Users_agoodman_github_CellProfi, __pyx_k_Users_agoodman_github_CellProfi, sizeof(__pyx_k_Users_agoodman_github_CellProfi), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_accumulator, __pyx_k_accumulator, sizeof(__pyx_k_accumulator), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_big_mask, __pyx_k_big_mask, sizeof(__pyx_k_big_mask), 0, 0, 1, 1},
  {&__pyx_n_s_bool, __pyx_k_bool, sizeof(__pyx_k_bool), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_s_centrosome__filter, __pyx_k_centrosome__filter, sizeof(__pyx_k_centrosome__filter), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_estimate, __pyx_k_estimate, sizeof(__pyx_k_estimate), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_ik, __pyx_k_ik, sizeof(__pyx_k_ik), 0, 0, 1, 1},
  {&__pyx_n_s_image_stride, __pyx_k_image_stride, sizeof(__pyx_k_image_stride), 0, 0, 1, 1},
  {&__pyx_n_s_imax, __pyx_k_imax, sizeof(__pyx_k_imax), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_istride, __pyx_k_istride, sizeof(__pyx_k_istride), 0, 0, 1, 1},
  {&__pyx_n_s_j, __pyx_k_j, sizeof(__pyx_k_j), 0, 0, 1, 1},
  {&__pyx_n_s_jk, __pyx_k_jk, sizeof(__pyx_k_jk), 0, 0, 1, 1},
  {&__pyx_n_s_jmax, __pyx_k_jmax, sizeof(__pyx_k_jmax), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_kernel, __pyx_k_kernel, sizeof(__pyx_k_kernel), 0, 0, 1, 1},
  {&__pyx_n_s_kernel_half_width, __pyx_k_kernel_half_width, sizeof(__pyx_k_kernel_half_width), 0, 0, 1, 1},
  {&__pyx_n_s_kernel_stride, __pyx_k_kernel_stride, sizeof(__pyx_k_kernel_stride), 0, 0, 1, 1},
  {&__pyx_n_s_kernel_width, __pyx_k_kernel_width, sizeof(__pyx_k_kernel_width), 0, 0, 1, 1},
  {&__pyx_n_s_kmax, __pyx_k_kmax, sizeof(__pyx_k_kmax), 0, 0, 1, 1},
  {&__pyx_n_s_kstride, __pyx_k_kstride, sizeof(__pyx_k_kstride), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mask, __pyx_k_mask, sizeof(__pyx_k_mask), 0, 0, 1, 1},
  {&__pyx_n_s_mask_offset, __pyx_k_mask_offset, sizeof(__pyx_k_mask_offset), 0, 0, 1, 1},
  {&__pyx_n_s_mask_stride, __pyx_k_mask_stride, sizeof(__pyx_k_mask_stride), 0, 0, 1, 1},
  {&__pyx_n_s_masked_convolution, __pyx_k_masked_convolution, sizeof(__pyx_k_masked_convolution), 0, 0, 1, 1},
  {&__pyx_n_s_median_filter, __pyx_k_median_filter, sizeof(__pyx_k_median_filter), 0, 0, 1, 1},
  {&__pyx_n_s_mstride, __pyx_k_mstride, sizeof(__pyx_k_mstride), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_output, __pyx_k_output, sizeof(__pyx_k_output), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_pa, __pyx_k_pa, sizeof(__pyx_k_pa), 0, 0, 1, 1},
  {&__pyx_n_s_paeth_decoder, __pyx_k_paeth_decoder, sizeof(__pyx_k_paeth_decoder), 0, 0, 1, 1},
  {&__pyx_n_s_pb, __pyx_k_pb, sizeof(__pyx_k_pb), 0, 0, 1, 1},
  {&__pyx_n_s_pc, __pyx_k_pc, sizeof(__pyx_k_pc), 0, 0, 1, 1},
  {&__pyx_n_s_percent, __pyx_k_percent, sizeof(__pyx_k_percent), 0, 0, 1, 1},
  {&__pyx_n_s_pimage, __pyx_k_pimage, sizeof(__pyx_k_pimage), 0, 0, 1, 1},
  {&__pyx_n_s_pixel_stride, __pyx_k_pixel_stride, sizeof(__pyx_k_pixel_stride), 0, 0, 1, 1},
  {&__pyx_n_s_pkernel, __pyx_k_pkernel, sizeof(__pyx_k_pkernel), 0, 0, 1, 1},
  {&__pyx_n_s_plane_number, __pyx_k_plane_number, sizeof(__pyx_k_plane_number), 0, 0, 1, 1},
  {&__pyx_n_s_pmask, __pyx_k_pmask, sizeof(__pyx_k_pmask), 0, 0, 1, 1},
  {&__pyx_n_s_poutput, __pyx_k_poutput, sizeof(__pyx_k_poutput), 0, 0, 1, 1},
  {&__pyx_n_s_ptr, __pyx_k_ptr, sizeof(__pyx_k_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_radius, __pyx_k_radius, sizeof(__pyx_k_radius), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_raster_count, __pyx_k_raster_count, sizeof(__pyx_k_raster_count), 0, 0, 1, 1},
  {&__pyx_n_s_raster_number, __pyx_k_raster_number, sizeof(__pyx_k_raster_number), 0, 0, 1, 1},
  {&__pyx_n_s_raster_stride, __pyx_k_raster_stride, sizeof(__pyx_k_raster_stride), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_uint32, __pyx_k_uint32, sizeof(__pyx_k_uint32), 0, 0, 1, 1},
  {&__pyx_n_s_uint8, __pyx_k_uint8, sizeof(__pyx_k_uint8), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 751; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 768; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "centrosome/_filter.pyx":768
 *                        <np.uint8_t *>mask.data,
 *                        <np.uint8_t *>output.data):
 *         raise MemoryError('Failed to allocate scratchpad memory')             # <<<<<<<<<<<<<<
 * 
 * @cython.boundscheck(False)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Failed_to_allocate_scratchpad_me); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 768; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "centrosome/_filter.pyx":732
 *     return 0
 * 
 * def median_filter(np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] output,
 */
  __pyx_tuple__8 = PyTuple_Pack(5, __pyx_n_s_data, __pyx_n_s_mask, __pyx_n_s_output, __pyx_n_s_radius, __pyx_n_s_percent); if (unlikely(!__pyx_tuple__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_agoodman_github_CellProfi, __pyx_n_s_median_filter, 732, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "centrosome/_filter.pyx":771
 * 
 * @cython.boundscheck(False)
 * def masked_convolution(np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                        np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                        np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] kernel):
 */
  __pyx_tuple__10 = PyTuple_Pack(23, __pyx_n_s_data, __pyx_n_s_mask, __pyx_n_s_kernel, __pyx_n_s_pkernel, __pyx_n_s_pimage, __pyx_n_s_pmask, __pyx_n_s_poutput, __pyx_n_s_kernel_stride, __pyx_n_s_image_stride, __pyx_n_s_mask_stride, __pyx_n_s_mask_offset, __pyx_n_s_kernel_width, __pyx_n_s_kernel_half_width, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ik, __pyx_n_s_jk, __pyx_n_s_istride, __pyx_n_s_mstride, __pyx_n_s_kstride, __pyx_n_s_accumulator, __pyx_n_s_big_mask, __pyx_n_s_output); if (unlikely(!__pyx_tuple__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(3, 0, 23, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_agoodman_github_CellProfi, __pyx_n_s_masked_convolution, 771, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "centrosome/_filter.pyx":837
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * def paeth_decoder(             # <<<<<<<<<<<<<<
 *     np.ndarray[dtype=np.uint8_t, ndim=3, negative_indices=False, mode='c'] x,
 *     np.int32_t raster_count):
 */
  __pyx_tuple__12 = PyTuple_Pack(21, __pyx_n_s_x, __pyx_n_s_raster_count, __pyx_n_s_raster_stride, __pyx_n_s_pixel_stride, __pyx_n_s_ptr, __pyx_n_s_a, __pyx_n_s_b, __pyx_n_s_c, __pyx_n_s_p, __pyx_n_s_pa, __pyx_n_s_pb, __pyx_n_s_pc, __pyx_n_s_estimate, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_imax, __pyx_n_s_jmax, __pyx_n_s_kmax, __pyx_n_s_raster_number, __pyx_n_s_plane_number); if (unlikely(!__pyx_tuple__12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(2, 0, 21, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_agoodman_github_CellProfi, __pyx_n_s_paeth_decoder, 837, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_filter(void); /*proto*/
PyMODINIT_FUNC init_filter(void)
#else
PyMODINIT_FUNC PyInit__filter(void); /*proto*/
PyMODINIT_FUNC PyInit__filter(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__filter(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_filter", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_centrosome___filter) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "centrosome._filter")) {
      if (unlikely(PyDict_SetItemString(modules, "centrosome._filter", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_ptype_10centrosome_7_filter_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_10centrosome_7_filter_ndarray)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "centrosome/_filter.pyx":1
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":23
 *     void *memset(void *, int, int)
 * 
 * import_array()             # <<<<<<<<<<<<<<
 * 
 * ##############################################################################
 */
  import_array();

  /* "centrosome/_filter.pyx":43
 * ##############################################################################
 * 
 * DTYPE_UINT32 = np.uint32             # <<<<<<<<<<<<<<
 * DTYPE_BOOL = np.bool
 * ctypedef np.uint16_t pixel_count_t
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_UINT32, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "centrosome/_filter.pyx":44
 * 
 * DTYPE_UINT32 = np.uint32
 * DTYPE_BOOL = np.bool             # <<<<<<<<<<<<<<
 * ctypedef np.uint16_t pixel_count_t
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_bool); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DTYPE_BOOL, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":732
 *     return 0
 * 
 * def median_filter(np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                   np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] output,
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_10centrosome_7_filter_1median_filter, NULL, __pyx_n_s_centrosome__filter); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_median_filter, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 732; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":771
 * 
 * @cython.boundscheck(False)
 * def masked_convolution(np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] data,             # <<<<<<<<<<<<<<
 *                        np.ndarray[dtype=np.uint8_t, ndim=2, negative_indices=False, mode='c'] mask,
 *                        np.ndarray[dtype=np.float64_t, ndim=2, negative_indices=False, mode='c'] kernel):
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_10centrosome_7_filter_3masked_convolution, NULL, __pyx_n_s_centrosome__filter); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_masked_convolution, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":837
 * @cython.boundscheck(False)
 * @cython.cdivision(True)
 * def paeth_decoder(             # <<<<<<<<<<<<<<
 *     np.ndarray[dtype=np.uint8_t, ndim=3, negative_indices=False, mode='c'] x,
 *     np.int32_t raster_count):
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_10centrosome_7_filter_5paeth_decoder, NULL, __pyx_n_s_centrosome__filter); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_paeth_decoder, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "centrosome/_filter.pyx":1
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * cimport cython
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../.python/virtualenvs/centrosome-2.7.10/lib/python2.7/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init centrosome._filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init centrosome._filter");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

static CYTHON_INLINE long __Pyx_mod_long(long a, long b) {
    long r = a % b;
    r += ((r != 0) & ((r ^ b) < 0)) * b;
    return r;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE int __Pyx_IsLittleEndian(void) {
  unsigned int n = 1;
  return *(unsigned char*)(&n) != 0;
}
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t < '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static CYTHON_INLINE PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number;
    int ndim = ctx->head->field->type->ndim;
;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
      case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if (ctx->enc_type == *ts && got_Z == ctx->is_complex &&
            ctx->enc_packmode == ctx->new_packmode) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}
static CYTHON_INLINE void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static CYTHON_INLINE int __Pyx_GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  if (obj == Py_None || obj == NULL) {
    __Pyx_ZeroBuffer(buf);
    return 0;
  }
  buf->buf = NULL;
  if (__Pyx_GetBuffer(obj, buf, flags) == -1) goto fail;
  if (buf->ndim != nd) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if ((unsigned)buf->itemsize != dtype->size) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_ZeroBuffer(buf);
  return -1;
}
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (info->buf == NULL) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_RaiseBufferFallbackError(void) {
  PyErr_SetString(PyExc_ValueError,
     "Buffer acquisition failed on assignment; and then reacquiring the old buffer failed too!");
}

static CYTHON_INLINE npy_intp __Pyx_div_npy_intp(npy_intp a, npy_intp b) {
    npy_intp q = a / b;
    npy_intp r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) return __pyx_pw_5numpy_7ndarray_1__getbuffer__(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) { __pyx_pw_5numpy_7ndarray_3__releasebuffer__(obj, view); return; }
    Py_DECREF(obj);
    view->obj = NULL;
}
#endif


      #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE npy_int32 __Pyx_PyInt_As_npy_int32(PyObject *x) {
    const npy_int32 neg_one = (npy_int32) -1, const_zero = (npy_int32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(npy_int32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(npy_int32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (npy_int32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_int32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(npy_int32, digit, digits[0])
                case 2:
                    if (8 * sizeof(npy_int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) >= 2 * PyLong_SHIFT) {
                            return (npy_int32) (((((npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) >= 3 * PyLong_SHIFT) {
                            return (npy_int32) (((((((npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) >= 4 * PyLong_SHIFT) {
                            return (npy_int32) (((((((((npy_int32)digits[3]) << PyLong_SHIFT) | (npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (npy_int32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(npy_int32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int32, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (npy_int32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(npy_int32, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(npy_int32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(npy_int32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_int32) (((npy_int32)-1)*(((((npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(npy_int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 2 * PyLong_SHIFT) {
                            return (npy_int32) ((((((npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(npy_int32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_int32) (((npy_int32)-1)*(((((((npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(npy_int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 3 * PyLong_SHIFT) {
                            return (npy_int32) ((((((((npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(npy_int32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_int32) (((npy_int32)-1)*(((((((((npy_int32)digits[3]) << PyLong_SHIFT) | (npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(npy_int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(npy_int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(npy_int32) - 1 > 4 * PyLong_SHIFT) {
                            return (npy_int32) ((((((((((npy_int32)digits[3]) << PyLong_SHIFT) | (npy_int32)digits[2]) << PyLong_SHIFT) | (npy_int32)digits[1]) << PyLong_SHIFT) | (npy_int32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(npy_int32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int32, long, PyLong_AsLong(x))
            } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(npy_int32, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            npy_int32 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (npy_int32) -1;
        }
    } else {
        npy_int32 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (npy_int32) -1;
        val = __Pyx_PyInt_As_npy_int32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to npy_int32");
    return (npy_int32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to npy_int32");
    return (npy_int32) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_int32(npy_int32 value) {
    const npy_int32 neg_one = (npy_int32) -1, const_zero = (npy_int32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(npy_int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(npy_int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_int32),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_uint8(npy_uint8 value) {
    const npy_uint8 neg_one = (npy_uint8) -1, const_zero = (npy_uint8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_uint8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_uint8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(npy_uint8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(npy_uint8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_uint8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_uint8),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
